import itertools
import os
import unittest
from collections import defaultdict
from copy import deepcopy
from pathlib import Path
from typing import Dict, Any
from unittest import mock

from parameterized import parameterized_class

# do not remove - prevents circular import
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector
from checkov.common.graph.db_connectors.rustworkx.rustworkx_db_connector import RustworkxConnector
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.runner_filter import RunnerFilter
from checkov.terraform import TFDefinitionKey
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck
from checkov.terraform.plan_runner import Runner, resource_registry
from checkov.terraform.plan_utils import get_entity_id


@parameterized_class([
    {"db_connector": NetworkxConnector},
    {"db_connector": RustworkxConnector},
])
class TestRunnerValid(unittest.TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        cls.orig_checks = deepcopy(resource_registry.checks)
        cls.orig_all_registered_checks = deepcopy(BaseCheckRegistry._BaseCheckRegistry__all_registered_checks)
        cls.db_connector = cls.db_connector

    def test_py_graph_check(self):
        if not self.db_connector == RustworkxConnector:
            return
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/py_graph_check_tf_plan"
        valid_dir_path_for_external_check = current_dir + '/py_check_tf_plan'
        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_99999']
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=[valid_dir_path_for_external_check],
                            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist))
        assert len(report.passed_checks) == 3

    def test_runner_two_checks_only(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ["CKV_AWS_21"]
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"], checks=checks_allowlist),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        for record in report.failed_checks:
            self.assertIn(record.check_id, checks_allowlist)
        self.assertEqual(report.get_summary()["failed"], 3)
        self.assertEqual(report.get_summary()["passed"], 3)

    def test_tf_plan_filtered_rule(self):
        if not self.db_connector == RustworkxConnector:
            return
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tf_plan_filtered_rule_success.json"
        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_300']
        report = runner.run(files=[valid_plan_path], runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist))
        assert len(report.passed_checks) == 1

    def test_tf_plan_filtered_rule(self):
        if not self.db_connector == RustworkxConnector:
            return
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tf_plan_filtered_rule_fail.json"
        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ['CKV_AWS_300']
        report = runner.run(files=[valid_plan_path], runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist))
        assert len(report.failed_checks) == 1

    def test_runner_record_severity(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        checks_allowlist = [custom_check_id]
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )

        self.assertEqual(report.failed_checks[0].severity, Severities[BcSeverities.LOW])

    def test_runner_check_severity_filter_omit(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        checks_allowlist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_check_severity_filter_include(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]
        checks_allowlist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_check_skip_filter_omit(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.LOW]
        checks_denylist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], skip_checks=checks_denylist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertFalse(any(c.check_id == custom_check_id for c in all_checks))

    def test_runner_check_skip_filter(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()

        custom_check_id = "MY_CUSTOM_CHECK"

        resource_registry.checks = defaultdict(list)

        class AnyFailingCheck(BaseResourceCheck):
            def __init__(self, *_, **__) -> None:
                super().__init__(
                    "this should fail",
                    custom_check_id,
                    [CheckCategories.ENCRYPTION],
                    ["aws_db_instance"]
                )

            def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
                return CheckResult.FAILED

        check = AnyFailingCheck()
        check.severity = Severities[BcSeverities.HIGH]
        checks_denylist = ['MEDIUM']
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], skip_checks=checks_denylist),
        )

        all_checks = report.failed_checks + report.passed_checks
        self.assertTrue(any(c.check_id == custom_check_id for c in all_checks))

    def test_plan_runner_with_empty_vpc_connection(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []

        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 107)

    def test_runner_child_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_with_child_modules/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 3)
        self.assertEqual(report.get_summary()["passed"], 5)

    def test_runner_nested_child_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_nested_child_modules/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=[current_dir + "/extra_yaml_checks"],
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 15)
        self.assertEqual(report.get_summary()["passed"], 3)

        failed_check_ids = set([c.check_id for c in report.failed_checks])
        expected_failed_check_ids = {
            "CKV_AWS_37",
            "CKV_AWS_38",
            "CKV_AWS_39",
            "CKV_AWS_58",
            "CUSTOM_GRAPH_AWS_1"
        }

        assert failed_check_ids == expected_failed_check_ids

        # reset graph checks
        runner.graph_registry.checks = []
        runner.graph_registry.load_checks()

    def test_runner_root_module_resources_no_values(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_root_module_resources_no_values/tfplan.json"
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        # 4 checks fail on test data for single eks resource as of present
        # If more eks checks are added then this number will need to increase correspondingly to reflect
        # This reasoning holds for all current pass/fails in these tests
        self.assertEqual(report.get_summary()["failed"], 4)
        self.assertEqual(report.get_summary()["passed"], 1)

        failed_check_ids = set([c.check_id for c in report.failed_checks])
        expected_failed_check_ids = {
            "CKV_AWS_37",
            "CKV_AWS_38",
            "CKV_AWS_39",
            "CKV_AWS_58",
        }

        assert failed_check_ids == expected_failed_check_ids

        # reset graph checks
        runner.graph_registry.checks = []
        runner.graph_registry.load_checks()

    def test_runner_root_module_resources_no_values_route53(self):
        #given
        plan_file = Path(__file__).parent / "resources/plan_root_module_resources_no_values/tfplan_route53.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(plan_file)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_38", "CKV2_AWS_39"]),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["passed"], 1)

        passed_check_ids = set(c.check_id for c in report.passed_checks)
        expected_passed_check_ids = {"CKV2_AWS_39"}

        self.assertCountEqual(passed_check_ids, expected_passed_check_ids)

    def test_runner_data_resource_partial_values(self):
        # In rare circumstances a data resource with partial values in the plan could cause false negatives
        # Often 'data' does not even appear in the *_modules[x].resources field within planned_values and is not scanned as expected
        # It can occur when tf module B depends on tf module A
        # And tf module A creates a resource that is used in a data block in tf module B
        # So some values can be known but other are not at plan time
        # This can cause the data block resource to be scanned as if it were a managed resource which is not configured correctly
        # See 'Modes': https://www.terraform.io/docs/internals/json-format.html#values-representation
        # This test verifies that such a circumstance stops occurring
        # There is a EKS Managed Resource and a EKS Data Resource
        # The EKS Managed Resource should have 4 failures corresponding with EKS checks.
        # The EKS Data Resource should not be scanned. Previously this would cause 8 failures.
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_data_resource_partial_values/tfplan.json"
        runner = Runner()
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 4)
        self.assertEqual(report.get_summary()["passed"], 1)

        failed_check_ids = set([c.check_id for c in report.failed_checks])
        expected_failed_check_ids = {
            "CKV_AWS_37",
            "CKV_AWS_38",
            "CKV_AWS_39",
            "CKV_AWS_58",
        }

        assert failed_check_ids == expected_failed_check_ids

    def test_runner_root_dir(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        root_dir = current_dir + "/resources"
        runner = Runner()
        report = runner.run(
            root_folder=root_dir, files=None, external_checks_dir=None, runner_filter=RunnerFilter(framework=["all"])
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 1)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertGreaterEqual(report.get_summary()["failed"], 71)
        self.assertGreaterEqual(report.get_summary()["passed"], 65)

        files_scanned = list(set(map(lambda rec: rec.file_path, report.failed_checks)))
        self.assertGreaterEqual(len(files_scanned), 6)

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        root_dir = current_dir + "/resources"
        runner = Runner()
        filter = RunnerFilter(framework=['terraform_plan'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.TERRAFORM_PLAN: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=root_dir, files=None, external_checks_dir=None, runner_filter=filter
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)

    def test_record_relative_path_with_relative_dir(self):
        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_dir_path = os.path.join(current_dir, "resources", "plan")

        # this is the relative path to the directory to scan (what would actually get passed to the -d arg)
        dir_rel_path = os.path.relpath(scan_dir_path).replace('\\', '/')

        runner = Runner()
        checks_allowlist = ["CKV_AWS_6"]
        report = runner.run(
            root_folder=dir_rel_path,
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        for record in all_checks:
            self.assertEqual(record.repo_file_path, f'/{os.path.join(dir_rel_path, record.file_path.lstrip("/"))}')

    def test_record_relative_path_with_relative_file(self):
        # this is just constructing the scan dir as normal
        current_dir = os.path.dirname(os.path.realpath(__file__))
        scan_file_path = os.path.join(current_dir, "resources", "plan", "tfplan.json")

        # this is the relative path to the file to scan (what would actually get passed to the -f arg)
        file_rel_path = os.path.relpath(scan_file_path)

        runner = Runner()
        checks_allowlist = ["CKV_AWS_20"]
        report = runner.run(
            root_folder=None,
            external_checks_dir=None,
            files=[file_rel_path],
            runner_filter=RunnerFilter(framework=["terraform"], checks=checks_allowlist),
        )

        all_checks = report.failed_checks + report.passed_checks
        for record in all_checks:
            self.assertEqual(record.repo_file_path, f'/{file_rel_path}')

    def test_runner_unexpected_eks_node_group_remote_access(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/unexpected/eks_node_group_remote_access.json"
        runner = Runner()
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"]),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 0)
        self.assertEqual(report.get_summary()["passed"], 1)

    def test_runner_with_resource_reference(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_with_resource_reference/tfplan.json"
        allowed_checks = ["CKV_AWS_84"]

        report = Runner().run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["all"], checks=allowed_checks),
        )

        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)
        self.assertIsNotNone(report.get_test_suite())
        self.assertEqual(report.get_exit_code({'soft_fail': False, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)
        self.assertEqual(report.get_exit_code({'soft_fail': True, 'soft_fail_checks': [], 'soft_fail_threshold': None, 'hard_fail_checks': [], 'hard_fail_threshold': None}), 0)

        self.assertEqual(report.get_summary()["failed"], 0)
        self.assertEqual(report.get_summary()["passed"], 1)

    def test_runner_with_resource_reference_graph_check(self):
        # given
        valid_plan_path = Path(__file__).parent / "resources/plan_with_resource_reference/tfplan_graph.json"
        allowed_checks = ["CKV2_AWS_6"]

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(valid_plan_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=allowed_checks),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["passed"], 1)

    def test_runner_with_resource_reference_extra_ref(self):
        # given
        valid_plan_path = Path(__file__).parent / "resources/plan_with_resource_reference/tfplan_extra_ref.json"
        extra_checks_dir_path = [str(Path(__file__).parent / "extra_tf_plan_checks")]
        allowed_checks = ["CUSTOM_CONNECTION_1"]

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(valid_plan_path)],
            external_checks_dir=extra_checks_dir_path,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=allowed_checks),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 4)

    def test_runner_skip_graph_when_no_plan_exists(self):
        # given
        tf_file_path = Path(__file__).parent / "resource/example/example.tf"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["passed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 0)

    def test_runner_utf_16_encoded(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_utf_16_encoding/tfplan.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        summary = report.get_summary()

        self.assertGreater(summary["failed"], 0)
        self.assertGreater(summary["passed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

    def test_runner_line_numbers(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_resource_reference/tfplan.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        failed_check = report.failed_checks[0]
        self.assertEqual(failed_check.file_line_range, [13, 19])

    def test_runner_ignore_lifecycle_checks(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_lifecycle_check/tfplan.json"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )

        # then
        self.assertEqual(len(report.failed_checks), 0)

    def test_runner_extra_check(self):
        # given
        current_dir = Path(__file__).parent
        tf_dir_path = str(current_dir / "resources/plan_with_deleted_resources")
        extra_checks_dir_path = [str(current_dir / "extra_tf_plan_checks")]

        # when
        report = Runner().run(
            root_folder=tf_dir_path,
            external_checks_dir=extra_checks_dir_path,
            runner_filter=RunnerFilter(checks=["CUSTOM_DELETE_1", "CUSTOM_DELETE_2"])
        )

        # then
        summary = report.get_summary()
        self.assertEqual(summary["failed"], 2)

        resource_ids = [check.resource for check in report.failed_checks]
        self.assertCountEqual(resource_ids,["aws_secretsmanager_secret.default", "aws_secretsmanager_secret.default"])

        # check also the details
        failed_check = next(check for check in report.failed_checks if check.check_id == "CUSTOM_DELETE_1")
        self.assertEqual(failed_check.details, ["some great details"])

    def test_runner_nested_child_modules_with_connections(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_nested_child_modules_with_connections/tfplan.json"

        passing_resources = {
            "module.s3_module.module.s3_submodule.aws_s3_bucket.submodule_bucket",
            "module.s3_module.aws_s3_bucket.module_bucket",
            "aws_s3_bucket.root_bucket",
        }
        failing_resources = {
            "module.s3_bucket.aws_s3_bucket.this[0]",
        }

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_6"]),
        )

        # then
        self.assertEqual(len(report.passed_checks), 3)
        self.assertEqual(len(report.failed_checks), 1)

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_runner_with_iam_policies(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_iam_policies/tfplan.json"

        passing_resources = {
            "aws_iam_policy.policy_pass",
        }
        failing_resources = {
            "aws_iam_role_policy.fail_1",
            "aws_iam_group_policy.fail_2",
            "aws_iam_user_policy.fail_3",
        }

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_40", "CKV_AWS_287"]),
        )

        # then
        summary = report.get_summary()

        self.assertEqual(summary["passed"], 2)  # "aws_iam_policy.policy_pass" passes both checks
        self.assertEqual(summary["failed"], 6)  # the rest fails both checks

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_runner_with_iam_data_block(self):
        # given
        tf_file_path = Path(__file__).parent / "resources/plan_with_iam_data_block/tfplan.json"

        failing_resources = {
            "data.aws_iam_policy_document.allow_access_from_another_account",
        }

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV_AWS_49"]),
        )

        # then
        summary = report.get_summary()
        self.assertEqual(summary["passed"], 0)
        self.assertEqual(summary["failed"], 1)

        failed_check_resources = {c.resource for c in report.failed_checks}
        self.assertEqual(failing_resources, failed_check_resources)

    @mock.patch.dict(os.environ, {'CHECKOV_EXPERIMENTAL_CROSS_VARIABLE_EDGES': 'True'})
    def test_plan_and_tf_combine_graph(self):
        tf_file_path = Path(__file__).parent / "resources/plan_and_tf_combine_graph/tfplan.json"

        repo_path = Path(__file__).parent / "resources/plan_and_tf_combine_graph"

        # deep_analysis disabled
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_6"], deep_analysis=False,
                                       repo_root_for_plan_enrichment=[repo_path])
        )

        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 2)

        # deep_analysis enabled
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_6"], deep_analysis=True, repo_root_for_plan_enrichment=[repo_path])
        )

        self.assertEqual(len(report.passed_checks), 2)
        self.assertEqual(len(report.failed_checks), 0)

        expected_addresses = ['aws_s3_bucket.example', 'aws_s3_bucket.example_2']
        report_addresses = [report.passed_checks[0].resource_address, report.passed_checks[1].resource_address]
        assert sorted(expected_addresses) == sorted(report_addresses)
        assert report.passed_checks[0].file_path.endswith('.json')
        assert report.passed_checks[1].file_path.endswith('.json')

    @mock.patch.dict(os.environ, {'EVAL_TF_PLAN_AFTER_UNKNOWN': 'True'})
    def test_plan_and_tf_combine_graph_with_missing_resources(self):
        tf_file_path = Path(__file__).parent / "resources/plan_and_tf_combine_graph_with_missing_resources/tfplan.json"
        repo_path = Path(__file__).parent / "resources/plan_and_tf_combine_graph_with_missing_resources"

        # deep_analysis disabled
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_61"], deep_analysis=False,
                                       repo_root_for_plan_enrichment=[repo_path])
        )

        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 2)

        # deep_analysis enabled
        report = Runner().run(
            root_folder=None,
            files=[str(tf_file_path)],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CKV2_AWS_61"], deep_analysis=True,
                                       repo_root_for_plan_enrichment=[repo_path])
        )

        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.failed_checks), 2)

        expected_addresses = ['aws_s3_bucket.example', 'aws_s3_bucket.example_2']
        report_addresses = [report.failed_checks[0].resource_address, report.failed_checks[1].resource_address]
        assert sorted(expected_addresses) == sorted(report_addresses)

    def test_plan_resources_ids(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_resources_ids_with_nested_modules/tfplan.json"
        valid_resources_ids = ["module.child_0.module.child_1_c.aws_eks_cluster.cluster",
                               "module.child_0.module.child_1_b.aws_eks_cluster.cluster",
                               "module.child_0.module.child_1_a.aws_eks_cluster.cluster"]
        runner = Runner()
        runner.graph_registry.checks = []
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=[current_dir + "/extra_yaml_checks"],
            runner_filter=RunnerFilter(framework=["terraform_plan"]),
        )
        self.assertGreater(report.get_summary()["failed"] + report.get_summary()["passed"], 0)

        for check in itertools.chain(report.failed_checks, report.passed_checks):
            self.assertIn(check.resource, valid_resources_ids)

        self.assertEqual(len(report.resources), 3)

    def test_plan_resources_created_by_modules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/extra_tf_plan_checks/modules.json"
        runner = Runner()
        report = runner.run(
            root_folder=None, external_checks_dir=None, files=[valid_plan_path],
            runner_filter=RunnerFilter(checks=['CKV2_GCP_12', 'CKV_GCP_88'])
        )
        passed_checks_CKV2_GCP_12 = [check for check in report.passed_checks if check.check_id == 'CKV2_GCP_12']
        passed_checks_CKV_GCP_88 = [check for check in report.passed_checks if check.check_id == 'CKV_GCP_88']

        assert passed_checks_CKV2_GCP_12[0].resource == 'module.achia_test_valid_443.google_compute_firewall.custom[0]'
        assert passed_checks_CKV2_GCP_12[1].resource == 'module.achia_test_valid_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV2_GCP_12[2].resource == 'module.achia_test_violating_no_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV2_GCP_12[3].resource == 'module.achia_test_violating_port.google_compute_firewall.custom[0]'

        assert passed_checks_CKV_GCP_88[0].resource == 'module.achia_test_valid_443.google_compute_firewall.custom[0]'
        assert passed_checks_CKV_GCP_88[1].resource == 'module.achia_test_valid_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV_GCP_88[2].resource == 'module.achia_test_violating_no_ports.google_compute_firewall.custom[0]'
        assert passed_checks_CKV_GCP_88[3].resource == 'module.achia_test_violating_port.google_compute_firewall.custom[0]'

    def test___get_file_path__with_tf_definition_key_uses_correct_file_path(self):
        tf_definition = TFDefinitionKey(file_path='test')
        file_path, scanned_file = Runner()._get_file_path(tf_definition, 'test')
        assert file_path == 'test'
        assert scanned_file == '/.'

    def test_plan_change_keys(self):
        # given
        current_dir = Path(__file__).parent
        tf_plan_path = current_dir / "resources/plan_change_keys/tfplan.json"
        external_checks_dir = current_dir / "extra_tf_plan_checks"

        # when
        report = Runner().run(
            root_folder=None,
            files=[str(tf_plan_path)],
            external_checks_dir=[str(external_checks_dir)],
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=["CUSTOM_CHANGE_1"]),
        )

        # then
        summary = report.get_summary()

        passing_resources = {
            'aws_security_group_rule.foo'
        }
        failing_resources = {
            'aws_security_group_rule.bar',
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        self.assertEqual(summary["resource_count"], 2)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_plan_with_providers(self):
        """
        Ensure AWS providers are parsed correctly and the credentials check runs against
        providers with aliases, too.
        """
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_plan_path = current_dir + "/resources/plan_with_providers/tfplan.json"
        runner = Runner(db_connector=self.db_connector())
        checks_allowlist = ["CKV_AWS_41"]
        report = runner.run(
            root_folder=None,
            files=[valid_plan_path],
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=["terraform_plan"], checks=checks_allowlist),
        )
        report_json = report.get_json()
        self.assertIsInstance(report_json, str)
        self.assertIsNotNone(report_json)

        for record in report.failed_checks:
            self.assertIn(record.check_id, checks_allowlist)
        self.assertEqual(report.get_summary()["failed"], 1)
        self.assertEqual(report.get_summary()["passed"], 2)

    def tearDown(self) -> None:
        resource_registry.checks = deepcopy(self.orig_checks)
        BaseCheckRegistry._BaseCheckRegistry__all_registered_checks = deepcopy(self.orig_all_registered_checks)

    def test_get_entity_id(self):
        resource_type_dict = {'__address__': 'azure.storage_use_azuread', '__end_line__': [14], '__start_line__': [0],
         'alias': ['storage_use_azuread'], 'end_line': [14], 'start_line': [0], 'storage_use_azuread': True}
        resource_name = "storage_use_azuread"
        assert get_entity_id(resource_type_dict, resource_name) == 'azure.storage_use_azuread'


if __name__ == "__main__":
    unittest.main()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import \
    BaseResourceCheck


class CVMUseDefaultSecurityGroup(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud CVM instances do not use the default security group "
        id = "CKV_TC_4"
        supported_resources = ['tencentcloud_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict) -> CheckResult:
        if conf.get("orderly_security_groups"):
            for osg in conf["orderly_security_groups"][0]:
                if ".default." in osg:
                    self.evaluated_keys = ["orderly_security_groups"]
                    return CheckResult.FAILED

        if conf.get("security_groups"):
            for sg in conf["security_groups"][0]:
                if ".default." in sg:
                    self.evaluated_keys = ["security_groups"]
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = CVMUseDefaultSecurityGroup()



import os
from unittest import mock

import responses
from checkov.common.models.enums import CheckResult


@responses.activate
@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_runner_verify_secrets_skip_invalid_suppressed(mock_bc_integration, mock_metadata_integration):
    current_dir = os.path.dirname(os.path.realpath(__file__))
    valid_dir_path = current_dir + "/resources/cfn"

    rel_resource_path = '/secret.yml'
    resource_id = '3472e46be802575792c8ddc3fcea5399a73078f1'
    verified_report = [
        {
            "violationId": "BC_GIT_2",
            "resourceId": f"{rel_resource_path}:{resource_id}",
            "status": "Invalid"
        }
    ]

    responses.add(
        method=responses.POST,
        url=f"{mock_bc_integration.api_url}/api/v1/secrets/reportVerification",
        json={'verificationReportSignedUrl': 'mock'},
        status=200
    )

    from checkov.runner_filter import RunnerFilter
    from checkov.secrets.runner import Runner
    runner = Runner()
    mock_bc_integration.persist_enriched_secrets = lambda x: 'mock'
    mock_bc_integration.bc_api_key = 'mock'
    runner.get_json_verification_report = lambda x: verified_report

    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], skip_checks=['Invalid']))

    assert len(report.skipped_checks) == 1
    assert report.skipped_checks[0].file_path == rel_resource_path
    assert report.skipped_checks[0].resource == resource_id
    assert report.skipped_checks[0].validation_status == 'Invalid'
    assert len(report.failed_checks) == 1
    assert report.failed_checks[0].validation_status != 'Invalid'


@responses.activate
@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_runner_verify_secrets_skip_all_no_effect(mock_bc_integration, mock_metadata_integration):
    current_dir = os.path.dirname(os.path.realpath(__file__))
    valid_dir_path = current_dir + "/resources/cfn"

    rel_resource_path = '/secret.yml'
    resource_id = '3472e46be802575792c8ddc3fcea5399a73078f1'
    second_resource_id = 'a8a2f5d0efa444d71973792b14df2e05c00458c4'
    verified_report = [
        {
            "violationId": "BC_GIT_2",
            "resourceId": f"{rel_resource_path}:{resource_id}",
            "status": "Invalid"
        },
        {
            "violationId": "BC_GIT_6",
            "resourceId": f"{rel_resource_path}:{second_resource_id}",
            "status": "Unknown"
        }
    ]

    responses.add(
        method=responses.POST,
        url=f"{mock_bc_integration.api_url}/api/v1/secrets/reportVerification",
        json={'verificationReportSignedUrl': 'mock'},
        status=200
    )

    from checkov.runner_filter import RunnerFilter
    from checkov.secrets.runner import Runner
    runner = Runner()
    mock_bc_integration.persist_enriched_secrets = lambda x: 'mock'
    mock_bc_integration.bc_api_key = 'mock'

    runner.get_json_verification_report = lambda x: verified_report

    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets'], skip_checks=['Invalid', 'Unknown', 'Valid']))

    assert len(report.skipped_checks) == 1
    assert report.skipped_checks[0].file_path == rel_resource_path
    assert report.skipped_checks[0].resource == resource_id
    assert report.skipped_checks[0].validation_status == 'Invalid'

    assert len(report.failed_checks) == 1
    assert report.failed_checks[0].file_path == rel_resource_path
    assert report.failed_checks[0].resource == second_resource_id
    assert report.failed_checks[0].validation_status == 'Unknown'


def test_modify_invalid_secrets_check_result_to_skipped(secrets_report_invalid_status) -> None:
    from checkov.secrets.runner import Runner
    Runner()._modify_invalid_secrets_check_result_to_skipped(secrets_report_invalid_status)

    assert len(secrets_report_invalid_status.failed_checks) == 0
    assert len(secrets_report_invalid_status.skipped_checks) == 4
    assert len(secrets_report_invalid_status.passed_checks) == 1

    assert all(check.check_result["result"] == CheckResult.SKIPPED
               for check in secrets_report_invalid_status.skipped_checks)
    assert all(check.check_result["suppress_comment"] == "Skipped invalid secret"
               for check in secrets_report_invalid_status.skipped_checks)





import os
from unittest import TestCase

from checkov.cloudformation.graph_builder.graph_components.block_types import BlockType
from checkov.cloudformation.graph_manager import CloudformationGraphManager
from checkov.cloudformation.parser import parse
from checkov.common.graph.db_connectors.networkx.networkx_db_connector import NetworkxConnector

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestCloudformationGraphManager(TestCase):
    def test_build_graph_from_source_directory_no_rendering(self):
        root_dir = os.path.realpath(os.path.join(TEST_DIRNAME, "./runner/resources"))
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, definitions = graph_manager.build_graph_from_source_directory(root_dir, render_variables=False,
                                                                                   excluded_paths=["skip.*", "double_state.*"])

        expected_resources_by_file = {
            os.path.join(root_dir, "no_properties.yaml"): [
                "AWS::Serverless::Function.NoPropertiesYaml"
            ],
            os.path.join(root_dir, "no_properties.json"): [
                "AWS::Serverless::Function.NoPropertiesJson"
            ],
            os.path.join(root_dir, "tags.yaml"): [
                "AWS::S3::Bucket.DataBucket",
                "AWS::S3::Bucket.NoTags",
                "AWS::EKS::Nodegroup.EKSClusterNodegroup",
                "AWS::AutoScaling::AutoScalingGroup.TerraformServerAutoScalingGroup",
            ],
            os.path.join(root_dir, "cfn_newline_at_end.yaml"): [
                "AWS::RDS::DBInstance.MyDB",
                "AWS::S3::Bucket.MyBucket",
            ],
            os.path.join(root_dir, "success.json"): [
                "AWS::S3::Bucket.acmeCWSBucket",
                "AWS::S3::Bucket.acmeCWSBucket2",
                "AWS::S3::BucketPolicy.acmeCWSBucketPolicy",
                "AWS::SNS::Topic.acmeCWSTopic",
                "AWS::SNS::TopicPolicy.acmeCWSTopicPolicy",
                "AWS::CloudTrail::Trail.acmeCWSTrail",
                "AWS::KMS::Key.CloudtrailKMSKey",
                "AWS::KMS::Alias.CloudtrailKMSKeyAlias",
                "AWS::SQS::Queue.acmeCWSQueue",
                "AWS::SQS::QueuePolicy.acmeCWSQueuePolicy",
                "AWS::SNS::Subscription.acmeCWSSubscription",
                "AWS::IAM::Role.acmeCWSSACrossAccountAccessRole",
                "AWS::EKS::Cluster.eksCluster",
                "Custom::acmeSnsCustomResource.acmeSnsCustomResource",
            ],
            os.path.join(root_dir, "fail.yaml"): [
                "AWS::SQS::Queue.UnencryptedQueue",
            ],
            os.path.join(root_dir, "graph.yaml"): [
                "AWS::AppSync::GraphQLApi.GoodAppSyncGraphQLApi",
                "AWS::WAFv2::WebACL.GoodWAFv2WebACL",
                "AWS::WAFv2::WebACLAssociation.WebACLAssociation",
                "AWS::AppSync::GraphQLApi.NoWAFAppSyncGraphQLApi"
            ],
            os.path.join(root_dir, "suppress_graph_check.yaml"): [
                "AWS::AppSync::GraphQLApi.CommentSuppress",
                "AWS::AppSync::GraphQLApi.MetadataSuppress"
            ]
        }
        self.assertEqual(49, len(local_graph.vertices))
        self.assertEqual(29, len(local_graph.vertices_by_block_type[BlockType.RESOURCE]))
        self.assertEqual(9, len(local_graph.vertices_by_block_type[BlockType.PARAMETERS]))
        self.assertEqual(6, len(local_graph.vertices_by_block_type[BlockType.OUTPUTS]))
        self.assertEqual(4, len(local_graph.vertices_by_block_type[BlockType.CONDITIONS]))
        self.assertEqual(1, len(local_graph.vertices_by_block_type[BlockType.MAPPINGS]))

        for v in local_graph.vertices:
            if v.block_type == BlockType.RESOURCE:
                self.assertIn(v.name, expected_resources_by_file[v.path])

        sqs_queue_vertex = local_graph.vertices[local_graph.vertices_block_name_map[BlockType.RESOURCE]["AWS::SQS::Queue.acmeCWSQueue"][0]]
        del sqs_queue_vertex.attributes['QueueName']['__file__']
        del sqs_queue_vertex.attributes['QueueName']['Fn::Join'][1][0]['__file__']
        self.assertDictEqual({'Fn::Join': ['', [{'Ref': 'ResourceNamePrefix', '__startline__': 650, '__endline__': 652}, '-acmecws']], '__startline__': 646, '__endline__': 656}, sqs_queue_vertex.attributes["QueueName"])

    def test_build_graph_from_source_directory_with_rendering(self):
        root_dir = os.path.realpath(os.path.join(TEST_DIRNAME, "./runner/resources"))
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph, definitions = graph_manager.build_graph_from_source_directory(root_dir, render_variables=True)

        sqs_queue_vertex = local_graph.vertices[local_graph.vertices_block_name_map[BlockType.RESOURCE]["AWS::SQS::Queue.acmeCWSQueue"][0]]
        del sqs_queue_vertex.config['QueueName']['__file__']
        expected_node = {'Fn::Join': ['', ['acme', '-acmecws']], '__startline__': 646, '__endline__': 656}
        self.assertDictEqual(expected_node, sqs_queue_vertex.config["QueueName"])
        found = False
        for d in definitions:
            if 'resources/success.json' in d:
                found = True
                node = definitions[d]['Resources']['acmeCWSQueue']['Properties']['QueueName']
                self.assertDictEqual(expected_node, node)
        self.assertTrue(found, 'Did not find the wanted node, for acmeCWSQueue')

    def test_build_graph_from_definitions(self):
        relative_file_path = "./checks/resource/aws/example_APIGatewayXray/APIGatewayXray-PASSED.yaml"
        definitions = {}
        file = os.path.realpath(os.path.join(TEST_DIRNAME, relative_file_path))
        (definitions[relative_file_path], definitions_raw) = parse(file)
        graph_manager = CloudformationGraphManager(db_connector=NetworkxConnector())
        local_graph = graph_manager.build_graph_from_definitions(definitions)
        self.assertEqual(1, len(local_graph.vertices))
        resource_vertex = local_graph.vertices[0]
        self.assertEqual("AWS::ApiGateway::Stage.Enabled", resource_vertex.name)
        self.assertEqual("AWS::ApiGateway::Stage.Enabled", resource_vertex.id)
        self.assertEqual(BlockType.RESOURCE, resource_vertex.block_type)
        self.assertEqual("CloudFormation", resource_vertex.source)
        self.assertDictEqual(definitions[relative_file_path]["Resources"]["Enabled"]["Properties"], resource_vertex.attributes)



import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.azure.StorageAccountMinimumTlsVersion import check


class TestAppServiceMinTLSVersion(unittest.TestCase):

    def test_failure_option_not_present(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              network_rules {
                default_action             = "Allow"
                ip_rules                   = ["100.0.0.1"]
                virtual_network_subnet_ids = [azurerm_subnet.example.id]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_insecure_option_present_tls10(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              min_tls_version          = "TLS1_0"
              network_rules {
                default_action             = "Allow"
                ip_rules                   = ["100.0.0.1"]
                virtual_network_subnet_ids = [azurerm_subnet.example.id]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_insecure_option_present_tls11(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              min_tls_version          = "TLS1_1"
              network_rules {
                default_action             = "Allow"
                ip_rules                   = ["100.0.0.1"]
                virtual_network_subnet_ids = [azurerm_subnet.example.id]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_secure_option_present(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              min_tls_version          = "TLS1_2"
              network_rules {
                default_action             = "Allow"
                ip_rules                   = ["100.0.0.1"]
                virtual_network_subnet_ids = [azurerm_subnet.example.id]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_future_option_present(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_storage_account" "example" {
              name                     = "example"
              resource_group_name      = data.azurerm_resource_group.example.name
              location                 = data.azurerm_resource_group.example.location
              account_tier             = "Standard"
              account_replication_type = "GRS"
              min_tls_version          = "TLS1_3"
              network_rules {
                default_action             = "Allow"
                ip_rules                   = ["100.0.0.1"]
                virtual_network_subnet_ids = [azurerm_subnet.example.id]
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_storage_account']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os
import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.data.aws.GithubActionsOIDCTrustPolicy import check
from checkov.terraform.runner import Runner
from checkov.terraform.plan_runner import Runner as PlanRunner


class TestGithubActionsOIDCTrustPolicy(unittest.TestCase):
    def test(self):
        test_files_dir = Path(__file__).parent / "example_GithubActionsOIDCTrustPolicy"

        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            "aws_iam_policy_document.pass1",
            "aws_iam_policy_document.pass2",
            "aws_iam_policy_document.pass3",
            "aws_iam_policy_document.pass-org-only",
            "aws_iam_policy_document.pass_aud_first",
            "aws_iam_policy_document.pass-gh-org",
        }
        failing_resources = {
            "aws_iam_policy_document.fail1",
            "aws_iam_policy_document.fail2",
            "aws_iam_policy_document.fail-wildcard",
            "aws_iam_policy_document.fail-abusable",
            "aws_iam_policy_document.fail-wildcard-assertion",
            "aws_iam_policy_document.fail-misused-repo",
        }

        passed_check_resources = set([c.resource for c in report.passed_checks])
        failed_check_resources = set([c.resource for c in report.failed_checks])

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

    def test_terraform_plan(self):
        runner = PlanRunner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_path = current_dir + "/example_GithubActionsOIDCTrustPolicy/tfplan.json"
        report = runner.run(files=[test_files_path], runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        passing_resources = {
            'module.poc.data.aws_iam_policy_document.r4["p1"]',
            'module.poc.data.aws_iam_policy_document.r4["p2"]',
            'module.poc.data.aws_iam_policy_document.r3["p1"]',
            'module.poc.data.aws_iam_policy_document.r3["p2"]',
        }

        passed_check_resources = set([c.resource for c in report.passed_checks])

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], 0)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)


if __name__ == "__main__":
    unittest.main()



import os

from lark import Tree

from checkov.terraform import TFDefinitionKey
from checkov.terraform.modules.module_utils import clean_parser_types
from checkov.terraform.tf_parser import TFParser
from unittest import TestCase

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


class TestParser(TestCase):
    def test_bool_parsing_avoid_remove_non_existing(self):
        conf = {'test': ['Bool'], 'variable': ['aws:SecureTransport'], 'values': [['false']]}
        actual = clean_parser_types(conf)
        expected = {'test': ['Bool'], 'variable': ['aws:SecureTransport'], 'values': [[False]]}
        self.assertDictEqual(expected, actual)

    def test_bool_parsing_sort_only_lists(self):
        conf = {'enabled_metrics': [['a', 'c', 'b'], 'b', 'a', 'c']}
        actual = clean_parser_types(conf)
        expected = {'enabled_metrics': [['a', 'b', 'c'], 'a', 'b', 'c']}
        self.assertDictEqual(expected, actual)

    def test_bool_parsing_sort_only_lists_with_bools(self):
        conf = {'enabled_metrics': [['a', 'true', 'false'], 'b', 'true', 'false']}
        actual = clean_parser_types(conf)
        expected = {'enabled_metrics': [[True, False, 'a'], True, False, 'b']}
        self.assertDictEqual(expected, actual)

    def test_set_parsing_to_list(self):
        conf = {'enabled_metrics': [['a', 'true', 'false'], 'b', 'true', 'false'], 'example_set': [{'1', '2', '3'}]}
        actual = clean_parser_types(conf)
        expected = {'enabled_metrics': [[True, False, 'a'], True, False, 'b'], 'example_set': [['1', '2', '3']]}
        self.assertDictEqual(expected, actual)

    def test_tree_parsing_to_str(self):
        conf = {'enabled_metrics': [['a', 'true', 'false'], 'b', 'true', 'false'], 'example_set': Tree("data", ["child1", "child2"])}
        actual = clean_parser_types(conf)
        expected = {'enabled_metrics': [[True, False, 'a'], True, False, 'b'], 'example_set': 'Tree(\'data\', [\'child1\', \'child2\'])'}
        self.assertDictEqual(expected, actual)

    def test_hcl_parsing_consistent_old_new(self):
        cur_dir = os.path.dirname(os.path.realpath(__file__))
        tf_dir = f'{cur_dir}/../resources/tf_parsing_comparison/tf_regular'
        old_tf_dir = f'{cur_dir}/../resources/tf_parsing_comparison/tf_old'
        _, tf_definitions = TFParser().parse_hcl_module(tf_dir, 'AWS')
        _, old_tf_definitions = TFParser().parse_hcl_module(old_tf_dir, 'AWS')
        definition_value = list(tf_definitions.values())[0]
        old_definition_value = list(tf_definitions.values())[0]
        self.assertDictEqual(definition_value, old_definition_value)

    def test_hcl_parsing_old_booleans_correctness(self):
        cur_dir = os.path.dirname(os.path.realpath(__file__))
        tf_dir = f'{cur_dir}/../resources/tf_parsing_comparison/tf_regular'
        _, tf_definitions = TFParser().parse_hcl_module(tf_dir, 'AWS')
        expected = [
            {
                "aws_cloudtrail": {
                    "tfer--cashdash_trail": {
                        "__end_line__": 11,
                        "__start_line__": 1,
                        "enable_log_file_validation": [True],
                        "enable_logging": [True],
                        "include_global_service_events": [True],
                        "is_multi_region_trail": [True],
                        "is_organization_trail": [False],
                        "kms_key_id": ["arn:aws:kms:us-east-1:098885917934:key/5e7c4a79-bd63-42ca-9ae0-8f8e41f9c2f1"],
                        "name": ["cashdash_trail"],
                        "s3_bucket_name": ["cashdash-trail"],
                        "sns_topic_name": ["arn:aws:sns:us-east-1:098885917934:clodtrail-sns-topic"],
                    }
                }
            },
            {
                "google_compute_instance": {
                    "tfer--sentry-002D-v1": {
                        "__end_line__": 67,
                        "__start_line__": 13,
                        "attached_disk": [
                            {
                                "device_name": ["sentry"],
                                "mode": ["READ_WRITE"],
                                "source": [
                                    "https://www.googleapis.com/compute/v1/projects/be-base-wksp-v1/zones/us-west3-b/disks/sentry-data-v1"
                                ],
                            }
                        ],
                        "boot_disk": [
                            {
                                "auto_delete": [True],
                                "device_name": ["persistent-disk-0"],
                                "initialize_params": [
                                    {
                                        "image": [
                                            "https://www.googleapis.com/compute/v1/projects/debian-cloud/global/images/debian-10-buster-v20200910"
                                        ],
                                        "size": ["10"],
                                        "type": ["pd-standard"],
                                    }
                                ],
                                "kms_key_self_link": [
                                    "projects/acme-project/locations/global/keyRings/global-v1/cryptoKeys/global-disk-key"
                                ],
                                "mode": ["READ_WRITE"],
                                "source": [
                                    "https://www.googleapis.com/compute/v1/projects/acme-project/zones/us-west3-b/disks/sentry-v1"
                                ],
                            }
                        ],
                        "can_ip_forward": [False],
                        "deletion_protection": [False],
                        "enable_display": [False],
                        "machine_type": ["n1-standard-2"],
                        "metadata": [{"block-project-ssh-keys": True, "some-other-attribute": False}],
                        "name": ["sentry-v1"],
                        "network_interface": [
                            {
                                "access_config": [{"nat_ip": ["34.106.48.192"], "network_tier": ["PREMIUM"]}],
                                "network": [
                                    "https://www.googleapis.com/compute/v1/projects/acme-project/global/networks/acme"
                                ],
                                "network_ip": ["10.40.0.53"],
                                "subnetwork": [
                                    "https://www.googleapis.com/compute/v1/projects/acme-project/regions/us-west3/subnetworks/sentry"
                                ],
                                "subnetwork_project": ["acme-project"],
                            }
                        ],
                        "project": ["acme-project"],
                        "scheduling": [
                            {"automatic_restart": [True], "on_host_maintenance": ["MIGRATE"], "preemptible": [False]}
                        ],
                        "service_account": [
                            {
                                "email": ["sentry-vm@acme-project.iam.gserviceaccount.com"],
                                "scopes": [
                                    [
                                        "https://www.googleapis.com/auth/devstorage.read_only",
                                        "https://www.googleapis.com/auth/logging.write",
                                        "https://www.googleapis.com/auth/monitoring.write",
                                        "https://www.googleapis.com/auth/userinfo.email",
                                    ]
                                ],
                            }
                        ],
                        "shielded_instance_config": [
                            {
                                "enable_integrity_monitoring": [True],
                                "enable_secure_boot": [False],
                                "enable_vtpm": [True],
                            }
                        ],
                        "tags": [["allow-sentry", "allow-ssh"]],
                        "zone": ["us-west3-b"],
                    }
                }
            },
        ]
        definition_key = TFDefinitionKey(file_path=os.path.join(tf_dir, "main.tf"), tf_source_modules=None)
        tf_definitions_resources = tf_definitions[definition_key]['resource']
        for index in range(len(tf_definitions_resources)):
            self.assertDictEqual(
                tf_definitions_resources[index],
                expected[index]
            )

    def test_hcl_parsing_sorting(self):
        source_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                   '../resources/tf_parsing_comparison/modifications_diff'))
        config_parser = TFParser()
        _, tf_definitions = config_parser.parse_hcl_module(source_dir, 'AWS')
        expected = ['https://www.googleapis.com/auth/devstorage.read_only', 'https://www.googleapis.com/auth/logging.write',
                    'https://www.googleapis.com/auth/monitoring.write', 'https://www.googleapis.com/auth/service.management.readonly',
                    'https://www.googleapis.com/auth/servicecontrol', 'https://www.googleapis.com/auth/trace.append']
        defintion_key = TFDefinitionKey(file_path=os.path.join(source_dir, "main.tf"), tf_source_modules=None)
        result_resource = tf_definitions[defintion_key]['resource'][0]['google_compute_instance']['tfer--test3']['service_account'][0]['scopes'][0]
        self.assertListEqual(result_resource, expected)

    def test_build_graph_with_linked_modules(self):
        source_dir = os.path.realpath(os.path.join(TEST_DIRNAME,
                                                   '../resources/nested_modules_double_call'))
        config_parser = TFParser()

        definitions = config_parser.parse_directory(source_dir)
        assert len(definitions.keys()) == 13
        assert '/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/main.tf' not in definitions
        assert '/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/third/main.tf[/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/main.tf#0]' not in definitions
        assert '/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/four/main.tf[/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/third/main.tf#0[/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/main.tf#0]]' not in definitions
        assert '/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/third/main.tf' not in definitions
        assert '/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/four/main.tf[/Users/arosenfeld/Desktop/dev/checkov/tests/terraform/graph/resources/nested_modules_double_call/third/main.tf#0]' not in definitions



import pytest

from checkov.common.output.record import Record


@pytest.mark.parametrize(
    "input_path,expected_path",
    [
        ("s3\\main.tf", "/s3/main.tf"),
        ("s3/main.tf", "/s3/main.tf"),
        ("/abs_path/to/s3/main.tf", "/abs_path/to/s3/main.tf"),
        ("../../s3/main.tf", "/s3/main.tf"),
    ],
    ids=["windows", "rel_path", "abs_path", "recursive_rel_path"],
)
def test_determine_repo_file_path(input_path: str, expected_path: str):
    assert Record._determine_repo_file_path(input_path) == expected_path


def test_from_reduced_json(json_reduced_check):
    # Act
    record = Record.from_reduced_json(json_reduced_check)

    # Assert
    assert record.check_id == 'CKV_GHA_1'
    assert record.check_name == 'Ensure ACTIONS_ALLOW_UNSECURE_COMMANDS isn\u0027t true on environment variables'
    assert record.check_result == {
            "result": "PASSED",
            "results_configuration": {}
        }
    assert record.resource == 'jobs(container-test-job)'
    assert record.file_path == '/.github/workflows/image_no_violation.yml'
    assert record.file_line_range == [7, 7]
    assert record.file_abs_path == '/tmp/checkov/elturgeman6/elturgeman/supplygoat1/main/src/.github/workflows/image_no_violation.yml'
    assert record.code_block == [
            [
                7,
                "    runs-on: ubuntu-latest\n"
            ],
        ]
    assert record.bc_check_id == 'BC_REPO_GITHUB_ACTION_1'


def test_code_line_string_strips_control_chars():
    """Embedded ANSI/OSC escape sequences in source lines should not be passed
    through to the CLI code-block output; tab and newline must be preserved."""
    code_block = [
        (1, 'resource "x" "y" {\n'),
        (2, '\tdescription = "\x1b[2J\x1b]8;;http://example/\x07link\x1b]8;;\x07"\n'),
        (3, '}\n'),
    ]

    out = Record._code_line_string(code_block, colorized=False)

    assert "\x1b" not in out
    assert "\x07" not in out
    # printable content, tab indentation and newlines are kept
    assert "link" in out
    assert "\tdescription" in out
    assert out.count("\n") == 3



import unittest

from checkov.common.util.secrets import string_has_secrets, ALL, AWS, GENERAL, omit_secret_value_from_line, \
    get_secrets_from_string


class TestSecrets(unittest.TestCase):

    def test_secrets(self):
        test_strings = [
            'AKIAIOSFODNN7EXAMPLE',  # checkov:skip=CKV_SECRET_2 test secret
            'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY',  # checkov:skip=CKV_SECRET_6 test secret
            '-----BEGIN RSA PRIVATE KEY-----\n',  # checkov:skip=CKV_SECRET_13 test secret
            'Hello from Bridgecrew',
            'cert-manager.io/secret: org/repo',
        ]

        # check that no category checks all
        self.assertEqual(3, sum(1 for s in test_strings if string_has_secrets(s)))

        # check one category
        self.assertEqual(2, sum(1 for s in test_strings if string_has_secrets(s, AWS)))

        # check two categories
        self.assertEqual(3, sum(1 for s in test_strings if string_has_secrets(s, AWS, GENERAL)))

        # check explicit all
        self.assertEqual(3, sum(1 for s in test_strings if string_has_secrets(s, ALL)))

        # check explicit all plus another category
        self.assertEqual(3, sum(1 for s in test_strings if string_has_secrets(s, ALL, AWS)))

    # Regression test for https://github.com/bridgecrewio/checkov/issues/754
    def test_does_not_consider_single_hash_as_a_secret(self):
        # SHA1
        self.assertFalse(string_has_secrets("b5a5b36b6be8d98c6f1bea655536d67abef23be8"))

        # MD5
        self.assertFalse(string_has_secrets("d9de48cf0676e9edb99bd8ee1ed44a21"))

    def test_omit_secret_value_from_line(self):
        secret = 'AKIAIOSFODNN7EXAMPLE'  # checkov:skip=CKV_SECRET_6 test secret
        line = 'access_key: "AKIAIOSFODNN7EXAMPLE"'

        censored_line = omit_secret_value_from_line(secret, line)

        self.assertEqual(censored_line, 'access_key: "AKIAI**********"')

    def test_omit_none_secret_from_line(self):
        line = 'text'
        self.assertEqual(line, omit_secret_value_from_line(secret=None, line_text=line))

    def test_omit_non_string_secret_from_line(self):
        line = 'text'
        secret = True

        self.assertEqual(line, omit_secret_value_from_line(secret, line))

    def test_omit_long_secret_value_from_line(self):
        secret = '123456AKIAIOSFODNN7EXAMPLEAKIAIOSFODNN7EXAMPLEAKIAIOSFODNN7EXAM'  # checkov:skip=CKV_SECRET_6 test secret
        line = 'access_key: "123456AKIAIOSFODNN7EXAMPLEAKIAIOSFODNN7EXAMPLEAKIAIOSFODNN7EXAM"'

        censored_line = omit_secret_value_from_line(secret, line)

        self.assertEqual(censored_line, 'access_key: "123456**********"')

    def test_get_secrets_from_secrets(self):
        s = 'access_key: "AKIAIOSFODNN7EXAMPLE"'

        secret = get_secrets_from_string(s)

        assert secret == ["AKIAIOSFODNN7EXAMPLE"]



import os
import unittest
import shutil

import hcl2

from checkov.terraform.modules.module_utils import validate_malformed_definitions, clean_bad_definitions, \
    clean_parser_types, serialize_definitions
from checkov.terraform.tf_parser import TFParser
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR


class ModuleTest(unittest.TestCase):

    def setUp(self) -> None:
        from checkov.terraform.module_loading.registry import ModuleLoaderRegistry

        # needs to be reset, because the cache belongs to the class not instance
        ModuleLoaderRegistry.module_content_cache = {}

        self.resources_dir = os.path.realpath(os.path.join(os.path.dirname(os.path.realpath(__file__)), "./resources"))
        self.external_module_path = ''

    def tearDown(self) -> None:
        if os.path.exists(self.external_module_path):
            shutil.rmtree(self.external_module_path)

    def test_module_double_slash_cleanup(self):
        with open(os.path.join(os.path.dirname(__file__), 'resources', 'double_slash.tf')) as f:
            tf = hcl2.load(f)
        non_malformed_definitions = validate_malformed_definitions(tf)
        definitions = {
            '/mock/path/to.tf': clean_bad_definitions(non_malformed_definitions)
        }
        module, _ = TFParser().parse_hcl_module_from_tf_definitions(definitions, '', 'terraform')
        print(module)
        self.assertEqual(1, len(module.blocks))
        self.assertEqual('ingress.annotations.kubernetes\\.io/ingress\\.class', module.blocks[0].attributes['set.name'])

    def test_module_double_slash_cleanup_string(self):
        tf = hcl2.loads("""
resource "helm_release" "test" {
  name       = "influxdb"
  repository = "https://helm.influxdata.com"
  chart      = "influxdb"
  namespace  = "influxdb"
  set {
    name  = "ingress.annotations.kubernetes\\.io/ingress\\.class"
    value = var.influxdb_ingress_annotations_kubernetes_ingress_class
  }
}
        """)
        non_malformed_definitions = validate_malformed_definitions(tf)
        definitions = {
            '/mock/path/to.tf': clean_bad_definitions(non_malformed_definitions)
        }
        module, _ = TFParser().parse_hcl_module_from_tf_definitions(definitions, '', 'terraform')
        print(module)
        self.assertEqual(1, len(module.blocks))
        self.assertEqual('ingress.annotations.kubernetes\\.io/ingress\\.class', module.blocks[0].attributes['set.name'])

    def test_module_with_resource_type_attribute(self):
        tf = hcl2.loads("""
resource "azurerm_security_center_subscription_pricing" "example" {
  tier = "free"
  resource_type = "VirtualMachines"
  extension {
    name = "ContainerRegistriesVulnerabilityAssessments"
  }
}
        """)
        non_malformed_definitions = validate_malformed_definitions(tf)
        definitions = {
            '/mock/path/to.tf': clean_bad_definitions(non_malformed_definitions)
        }
        module, _ = TFParser().parse_hcl_module_from_tf_definitions(definitions, '', 'terraform')
        self.assertEqual(1, len(module.blocks))
        self.assertEqual(['VirtualMachines'], module.blocks[0].attributes['_resource_type'])

    def test_parse_hcl_module_serialize_definitions(self):
        parser = TFParser()
        directory = os.path.join(self.resources_dir, "parser_nested_modules")
        self.external_module_path = os.path.join(directory, DEFAULT_EXTERNAL_MODULES_DIR)
        tf_definitions = parser.parse_directory(directory=directory, out_evaluations_context={})
        tf_definitions = clean_parser_types(tf_definitions)
        tf_definitions_encoded = serialize_definitions(tf_definitions)
        self.assertEqual(tf_definitions_encoded, tf_definitions)



import unittest

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as policy_metadata_integration
from checkov.common.bridgecrew.integration_features.features.custom_policies_integration import integration as custom_policies_integration
from checkov.common.bridgecrew.integration_features.features.fixes_integration import integration as fixes_integration
from checkov.common.bridgecrew.integration_features.features.repo_config_integration import integration as repo_config_integration
from checkov.common.bridgecrew.integration_features.features.suppressions_integration import integration as suppressions_integration


class TestSuppressionsIntegration(unittest.TestCase):
    def test_feature_order(self):
        self.assertGreater(fixes_integration.order, max([i.order for i in [policy_metadata_integration, custom_policies_integration, repo_config_integration, suppressions_integration]]))
        self.assertGreater(custom_policies_integration.order, policy_metadata_integration.order)
        self.assertGreater(suppressions_integration.order, policy_metadata_integration.order)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.DocDBTLS import check


class TestDocDBTLS(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_docdb_cluster_parameter_group" "test" {
                  family      = "docdb3.6"
                  name        = "test"
                  description = "docdb cluster parameter group"

                  parameter {
                    name  = "tls"
                    value = "disabled"
                  }

                  parameter {
                    name  = "other-param"
                    value = "enabled"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_docdb_cluster_parameter_group']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_no_parameters(self):
        hcl_res = hcl2.loads("""
                resource "aws_docdb_cluster_parameter_group" "test" {
                  family      = "docdb3.6"
                  name        = "test"
                  description = "docdb cluster parameter group"
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_docdb_cluster_parameter_group']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_with_parameters(self):
        hcl_res = hcl2.loads("""
                resource "aws_docdb_cluster_parameter_group" "test" {
                  family      = "docdb3.6"
                  name        = "test"
                  description = "docdb cluster parameter group"

                  parameter {
                    name  = "tls"
                    value = "enabled"
                  }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_docdb_cluster_parameter_group']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



from pathlib import Path

from checkov.arm.parser.parser import load, parse

EXAMPLES_DIR = Path(__file__).parent / "examples"

def test_load_mariadb():
    # given
    file_path = EXAMPLES_DIR / "json/mariadb.json"

    # when
    template, file_lines = load(file_path)

    # then
    assert template["$schema"] == "https://schema.management.azure.com/schemas/2019-04-01/deploymentTemplate.json#"
    assert template["contentVersion"] == "1.0.0.0"
    assert len(file_lines) == 25


def test_load_not_arm_file():
    # given
    file_path = EXAMPLES_DIR / "json/normal.json"

    # when
    template, file_lines = load(file_path)

    # then
    assert template == {}
    assert file_lines == []


def test_parse_arm_file_with_comments():
    # given
    file_path = EXAMPLES_DIR / "json/with_comments.json"

    # when
    template, file_lines = parse(str(file_path))

    # then
    assert template is None
    assert file_lines is None



import responses
import pytest
from unittest import mock

from checkov.terraform.module_loading.loaders.registry_loader import RegistryLoader
from checkov.terraform.module_loading.module_params import ModuleParams


@responses.activate
def test_module_version_url_invoked_once():
    # given
    module_version_url = "https://registry.terraform.io/v1/modules/terraform-aws-modules/example/versions"
    responses.add(
        method=responses.GET,
        url=module_version_url,
        json={"modules": [{"versions": [{"version": "1.0.0"}]}]},
        status=200,
    )
    loader = RegistryLoader()
    RegistryLoader.modules_versions_cache = {}  # reset cache
    module_params = ModuleParams("", "", "terraform-aws-modules/example", "", "", "")
    loader.discover(module_params)

    # when
    loader._is_matching_loader(module_params)
    loader._is_matching_loader(module_params)

    # then
    responses.assert_call_count(module_version_url, 1)
    assert loader.modules_versions_cache == {module_version_url: ["1.0.0"]}

def test_determine_tf_api_endpoints_tfc():
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "terraform-aws-modules/example", "", "", "")
    loader.discover(module_params)

    # when
    loader._determine_tf_api_endpoints(module_params)

    # then
    assert module_params.tf_host_name == "app.terraform.io"
    assert module_params.tf_modules_endpoint == "https://registry.terraform.io/v1/modules/"
    assert module_params.tf_modules_versions_endpoint == "https://registry.terraform.io/v1/modules/terraform-aws-modules/example/versions"

@pytest.mark.parametrize(
    "discovery_response",
    [
        ({
        "modules.v1": "/api/registry/v1/modules/",
        "providers.v1": "/api/registry/v1/providers/",
        "state.v2": "/api/v2/",
        "tfe.v2": "/api/v2/",
        "tfe.v2.1": "/api/v2/",
        "tfe.v2.2": "/api/v2/",
        "versions.v1": "https://checkpoint-api.hashicorp.com/v1/versions/"
        }),
        ({
        "modules.v1": "https://example.registry.com/api/registry/v1/modules/",
        "providers.v1": "https://example.registry.com/api/registry/v1/providers/",
        "state.v2": "https://example.registry.com/api/v2/",
        "tfe.v2": "https://example.registry.com/api/v2/",
        "tfe.v2.1": "https://example.registry.com/api/v2/",
        "tfe.v2.2": "https://example.registry.com/api/v2/",
        "versions.v1": "https://checkpoint-api.hashicorp.com/v1/versions/"
        }),
    ]
)
@responses.activate
def test_determine_tf_api_endpoints_tfe(discovery_response):
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "example.registry.com/terraform-aws-modules/example", "", "", "")
    with mock.patch.dict("os.environ", {"TF_HOST_NAME": "example.registry.com"}):
        loader.discover(module_params)
    responses.add(
        method=responses.GET,
        url=f"https://{module_params.tf_host_name}/.well-known/terraform.json",
        json=discovery_response,
        status=200,
    )

    # when
    loader._determine_tf_api_endpoints(module_params)

    # then
    responses.assert_call_count(f"https://{module_params.tf_host_name}/.well-known/terraform.json", 1)
    assert module_params.tf_host_name == "example.registry.com"
    assert module_params.tf_modules_endpoint == "https://example.registry.com/api/registry/v1/modules/"
    assert module_params.tf_modules_versions_endpoint == "https://example.registry.com/api/registry/v1/modules/terraform-aws-modules/example/versions"

@responses.activate
def test_load_module():
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "terraform-aws-modules/example", "", "", "")
    module_params.tf_modules_endpoint = "https://example.registry.com/api/registry/v1/modules/"
    module_params.best_version = "1.0.0"
    with mock.patch.dict("os.environ", {"TF_HOST_NAME": "example.registry.com"}):
        loader.discover(module_params)
    responses.add(
        method=responses.GET,
        url="https://example.registry.com/api/registry/v1/modules/terraform-aws-modules/example/1.0.0/download",
        status=200,
    )

    # when
    loader._load_module(module_params)

    # then
    responses.assert_call_count("https://example.registry.com/api/registry/v1/modules/terraform-aws-modules/example/1.0.0/download", 1)

@pytest.mark.parametrize(
    "download_url, expected_result",
    [
        ("https://example.com/download?archive=tgz", "tgz"),
        ("https://example.com/download?archive=zip", "zip"),
        ("https://example.com/download/module.zip", "zip"),
        ("https://example.com/download/module.zip?sig=foo", "zip"),
        ("https://example.com/download/module/archive", None),
    ]
)
def test_get_archive_extension(download_url, expected_result):
    archive_extension = RegistryLoader._get_archive_extension(download_url)
    assert archive_extension == expected_result

@pytest.mark.parametrize(
    "tf_host_name, module_download_url, expected_result",
    [
        ("example.com", "https://example.com/download?archive=tgz", "https://example.com/download?archive=tgz"),
        ("example.com", "https://example.com/abc", "https://example.com/abc"),
        ("example.com", "/api/registry/v1/modules/namespace/version/download?archive=tgz", "https://example.com/api/registry/v1/modules/namespace/version/download?archive=tgz"),
    ]
)
def test_normalize_module_download_url(tf_host_name, module_download_url, expected_result):
    # given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", "example.com/terraform-aws-modules/example", "", "", "")
    with mock.patch.dict("os.environ", {"TF_HOST_NAME": tf_host_name}):
        loader.discover(module_params)

    # when
    normalized_url = loader._normalize_module_download_url(module_params, module_download_url)

    # then
    assert normalized_url == expected_result

@pytest.mark.parametrize(
    "source_url",
    [
        ("git::https://example.com/repo.git"),
        ("git@github.com:org/repo"),
        ("github.com/org/repo"),
        ("bitbucket.org/org/repo"),
    ]
)
def test_is_matching_loader_git_sources(source_url):
    #given
    loader = RegistryLoader()
    module_params = ModuleParams("", "", source_url, "", "", "")
    loader.discover(module_params)

    # then
    assert not loader._is_matching_loader(module_params)


def test_load_module_returns_inner_module_path_when_dest_dir_exists(tmp_path):
    """
    When dest_dir already exists (e.g. a previous module from the same base source
    was already downloaded) and inner_module is set, _load_module should return
    the inner module subdirectory, not just the base dest_dir.

    This is the scenario that caused only the first set of inner modules to be
    scanned on Linux, while subsequent inner modules from the same base source
    were silently skipped.
    """
    # given
    loader = RegistryLoader()
    base_dir = tmp_path / "base_module"
    inner_module_a = base_dir / "modules" / "asg"
    inner_module_b = base_dir / "modules" / "ssm_document"

    # Simulate a previously downloaded module: create the base dir and both inner module dirs
    inner_module_a.mkdir(parents=True)
    inner_module_b.mkdir(parents=True)

    # First module: modules/asg
    module_params_a = ModuleParams(
        root_dir=str(tmp_path),
        current_dir=str(tmp_path),
        source="example.com/org/repo//modules/asg",
        source_version="1.0.0",
        dest_dir=str(base_dir),
        external_modules_folder_name=".external_modules",
        inner_module="modules/asg",
    )
    module_params_a.best_version = "1.0.0"

    # Second module: modules/ssm_document (different inner module, same base)
    module_params_b = ModuleParams(
        root_dir=str(tmp_path),
        current_dir=str(tmp_path),
        source="example.com/org/repo//modules/ssm_document",
        source_version="1.0.0",
        dest_dir=str(base_dir),
        external_modules_folder_name=".external_modules",
        inner_module="modules/ssm_document",
    )
    module_params_b.best_version = "1.0.0"

    # when
    content_a = loader._load_module(module_params_a)
    content_b = loader._load_module(module_params_b)

    # then - both should return their respective inner module paths
    assert content_a.loaded()
    assert content_a.path() == str(inner_module_a)

    assert content_b.loaded()
    assert content_b.path() == str(inner_module_b)

    # and they should be different paths
    assert content_a.path() != content_b.path()


def test_load_module_returns_dest_dir_when_no_inner_module(tmp_path):
    """
    When dest_dir already exists and there is no inner_module,
    _load_module should return dest_dir as before (no regression).
    """
    # given
    loader = RegistryLoader()
    base_dir = tmp_path / "base_module"
    base_dir.mkdir(parents=True)

    module_params = ModuleParams(
        root_dir=str(tmp_path),
        current_dir=str(tmp_path),
        source="example.com/org/repo",
        source_version="1.0.0",
        dest_dir=str(base_dir),
        external_modules_folder_name=".external_modules",
    )
    module_params.best_version = "1.0.0"

    # when
    content = loader._load_module(module_params)

    # then
    assert content.loaded()
    assert content.path() == str(base_dir)



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.CognitiveServicesDisablesPublicNetwork import check
from checkov.common.models.enums import CheckResult


class TestCognitiveServicesDisablesPublicNetwork(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_cognitive_account" "examplea" {
  name                = "example-account"
  location            = var.resource_group.location
  resource_group_name = var.resource_group.name
  kind                = "Face"

  public_network_access_enabled = true

  sku_name = "S0"
}
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_cognitive_account']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_missing_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_cognitive_account" "examplea" {
  name                = "example-account"
  location            = var.resource_group.location
  resource_group_name = var.resource_group.name
  kind                = "Face"

  sku_name = "S0"
}
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_cognitive_account']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)
        
    def test_success(self):
        hcl_res = hcl2.loads("""
resource "azurerm_cognitive_account" "examplea" {
  name                = "example-account"
  location            = var.resource_group.location
  resource_group_name = var.resource_group.name
  kind                = "Face"

  public_network_access_enabled = false

  sku_name = "S0"
}
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_cognitive_account']['examplea']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import itertools
import json
import os
import sys
import unittest

current_dir = os.path.dirname(os.path.realpath(__file__))


class TestCheckovJsonReport(unittest.TestCase):

    def test_terragoat_report(self):
        report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_terragoat.json')
        self.validate_report(os.path.abspath(report_path))

    def test_kustomizegoat_report(self):
        if not sys.platform.startswith('win'):
            report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_kustomizegoat.json')
            self.validate_report(os.path.abspath(report_path))

    def test_cfngoat_report(self):
        report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_cfngoat.json')
        self.validate_report(os.path.abspath(report_path))
        # self.validate_check_in_report(report_path, "CKV2_AWS_26")

    def test_k8goat_report(self):
        report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_kubernetes-goat.json')
        self.validate_report(os.path.abspath(report_path))

    def test_k8goat_report_not_win(self):
        if not sys.platform.startswith('win'):
            report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_kubernetes-goat-helm.json')
            self.validate_report(os.path.abspath(report_path))

    def test_checkov_report_terragoat_with_skip(self):
        report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_terragoat_with_skip.json')
        checkov2_graph_findings = 0
        with open(report_path) as json_file:
            data = json.load(json_file)
            for check_result in data["results"]["passed_checks"]:
                self.assertNotEqual(check_result["check_id"], "CKV_AWS_33")
                self.assertNotEqual(check_result["check_id"], "CKV_AWS_41")
                if check_result["check_id"].startswith('CKV2'):
                    checkov2_graph_findings += 1
        # self.assertGreater(checkov2_graph_findings, 5)  # Commented out as it's causing failures and might be outdated

    def validate_report(self, report_path):
        with open(report_path) as json_file:
            data = json.load(json_file)
            if isinstance(data, list):
                for framework_report in data:
                    self.validate_report_not_empty(framework_report)
            else:
                self.validate_report_not_empty(data)

    def validate_report_not_empty(self, report):
        self.assertEqual(report["summary"]["parsing_errors"], 0,
                         f"expecting 0 parsing errors but got: {report['results']['parsing_errors']}")
        self.assertGreater(report["summary"]["failed"], 1,
                           f"expecting more than 1 failed checks, got: {report['summary']['failed']}")
        self.assertGreater(report['results']['failed_checks'][0]['file_line_range'][1], 0)

    def validate_json_quiet(self):
        report_path = os.path.join(os.path.dirname(current_dir), 'checkov_report_cfngoat_quiet.json')
        with open(report_path) as json_file:
            data = json.load(json_file)
            self.assertTrue(data["results"]["failed_checks"])
            self.assertFalse(data["results"]["passed_checks"])
            self.assertTrue(data["summary"])

    def validate_check_in_report(self, report_path, check_id):
        with open(report_path) as json_file:
            data = json.load(json_file)[0]
        assert any(check["check_id"] == check_id for check in
                   itertools.chain(data["results"]["failed_checks"], data["results"]["passed_checks"]))


if __name__ == '__main__':
    unittest.main()



from typing import Any, List

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceSlotMinTLS(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure the App service slot is using the latest version of TLS encryption"
        id = "CKV_AZURE_154"
        supported_resources = ['azurerm_app_service_slot']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "site_config/[0]/min_tls_version/[0]"

    def get_expected_value(self):
        return '1.2'

    def get_expected_values(self) -> List[Any]:
        return ["1.2", 1.2, "1.3", 1.3]


check = AppServiceSlotMinTLS()



from aws_cdk import core
from aws_cdk import aws_autoscaling as autoscaling

class MyAutoScalingLaunchConfig(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Launch Configuration
        launch_config = autoscaling.CfnLaunchConfiguration(
            self, 'MyLaunchConfiguration',
            image_id='ami-12345678',  # Replace with your desired AMI ID
            instance_type='t2.micro',  # Replace with your desired instance type
            block_device_mappings=[{
                'deviceName': '/dev/xvda',
                'ebs': {
                    'encrypted': True
                }
            }]
            # Other properties for your Launch Configuration
        )

app = core.App()
MyAutoScalingLaunchConfig(app, "MyAutoScalingLaunchConfig")
app.synth()

class MyAutoScalingLaunchConfig(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Launch Configuration
        launch_config = autoscaling.CfnLaunchConfiguration(
            self, 'MyLaunchConfiguration',
            image_id='ami-12345678',  # Replace with your desired AMI ID
            instance_type='t2.micro',  # Replace with your desired instance type
            block_device_mappings=[{
                'deviceName': '/dev/xvda',
            }]
            # Other properties for your Launch Configuration
        )

app = core.App()
MyAutoScalingLaunchConfig(app, "MyAutoScalingLaunchConfig")
app.synth()




import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.VPCDefaultNetwork import check


class TestDefaultVPC(unittest.TestCase):

    def test_failure(self):
        """
          When there is a resource with aws_default_vpc, it should fail whatever the config is.
        """
        hcl_res = hcl2.loads("""
        resource "aws_default_vpc" "default" {
            tags = {
                Name = "Default VPC"
            }
        }
        """)

        resource_conf = hcl_res['resource'][0]['aws_default_vpc']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_config(self):
        """
        There is no success/pass scenario for this resource as we want to avoid the creation of this resource.
        """
        resource_conf = {
            "enable_dns_support": "true"
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class StorageSyncServicePermissiveAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure Storage Sync Service is not configured with overly permissive network access"
        id = "CKV_AZURE_250"
        supported_resources = ['azurerm_storage_sync']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, Any]) -> CheckResult:
        if "incoming_traffic_policy" in conf:
            if "AllowAllTraffic" in conf["incoming_traffic_policy"]:
                return CheckResult.FAILED
        else:
            return CheckResult.FAILED

        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ['incoming_traffic_policy']


check = StorageSyncServicePermissiveAccess()



import unittest

from checkov.common.util.type_forcers import convert_prisma_policy_filter_to_params


class TestTypeForcers(unittest.TestCase):
    def test_convert_prisma_policy_filter_to_dict(self):
        self.assertListEqual(convert_prisma_policy_filter_to_params('F1=A,F2=B'), [('F1', 'A'), ('F2', 'B')])
        self.assertListEqual(convert_prisma_policy_filter_to_params(''), [])
        self.assertListEqual(convert_prisma_policy_filter_to_params(None), [])
        self.assertListEqual(convert_prisma_policy_filter_to_params('F1 =   A,   F2= B '), [('F1', 'A'), ('F2', 'B')])
        self.assertListEqual(convert_prisma_policy_filter_to_params('F1=A,B,F2=C'), [('F1', 'A'), ('F2', 'C')])
        self.assertListEqual(convert_prisma_policy_filter_to_params('F1=A,F2=B,C'), [('F1', 'A'), ('F2', 'B')])
        self.assertListEqual(convert_prisma_policy_filter_to_params('F1=A,F2=B,F1=C'), [('F1', 'A'), ('F2', 'B'), ('F1', 'C')])
        self.assertListEqual(convert_prisma_policy_filter_to_params('F1=A,F2=B,F1=C,F1=DDD'), [('F1', 'A'), ('F2', 'B'), ('F1', 'C'), ('F1', 'DDD')])

        policy_string = 'policy.name=AWS S3 bucket ACL grants READ permission to everyone'
        filter_string = convert_prisma_policy_filter_to_params(policy_string)
        self.assertListEqual(filter_string, [('policy.name', 'AWS S3 bucket ACL grants READ permission to everyone')])


if __name__ == '__main__':
    unittest.main()



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.NeptuneClusterInstancePublic import check
from checkov.common.models.enums import CheckResult

class TestNeptuneClusterInstancePublic(unittest.TestCase):

    def test_success(self):
        hcl_res = hcl2.loads("""
resource "aws_neptune_cluster_instance" "example" {
  count              = 2
  cluster_identifier = aws_neptune_cluster.default.id
  engine             = "neptune"
  instance_class     = "db.r4.large"
  apply_immediately  = true
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_neptune_cluster_instance']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_explicit(self):
        hcl_res = hcl2.loads("""
resource "aws_neptune_cluster_instance" "example" {
  count               = 2
  cluster_identifier  = aws_neptune_cluster.default.id
  engine              = "neptune"
  instance_class      = "db.r4.large"
  apply_immediately   = true
  publicly_accessible = false
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_neptune_cluster_instance']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads("""
resource "aws_neptune_cluster_instance" "example" {
  count               = 2
  cluster_identifier  = aws_neptune_cluster.default.id
  engine              = "neptune"
  instance_class      = "db.r4.large"
  apply_immediately   = true
  publicly_accessible = true
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_neptune_cluster_instance']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import copy
import os
import warnings
from typing import List

from parameterized import parameterized_class

from checkov.cloudformation.graph_manager import CloudformationGraphManager
from tests.graph_utils.utils import set_db_connector_by_graph_framework, PARAMETERIZED_GRAPH_FRAMEWORKS
from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.models.enums import CheckResult
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from tests.common.graph.checks.test_yaml_policies_base import TestYamlPoliciesBase


file_dir = os.path.dirname(__file__)


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestYamlPolicies(TestYamlPoliciesBase):
    def __init__(self, args):
        db_connector = set_db_connector_by_graph_framework(self.graph_framework)
        graph_manager = CloudformationGraphManager(db_connector=db_connector)
        super().__init__(graph_manager,
                         os.path.abspath(os.path.join(file_dir, "../../../../checkov/cloudformation/checks/graph_checks")),
                         os.path.join(file_dir, "test_checks"), "cloudformation", __file__, args)

    def setUp(self) -> None:
        warnings.filterwarnings("ignore", category=ResourceWarning)
        warnings.filterwarnings("ignore", category=DeprecationWarning)

    def test_SagemakerNotebookEncryption(self):
        self.go("SagemakerNotebookEncryption")

    def test_MSKClusterLogging(self):
        self.go("MSKClusterLogging")

    def test_LambdaFunction(self):
        self.go("LambdaFunction")

    def test_SageMakerIAMPolicyOverlyPermissiveToAllTraffic(self):
        self.go("SageMakerIAMPolicyOverlyPermissiveToAllTraffic")

    def test_ALBRedirectHTTPtoHTTPS(self):
        self.go("ALBRedirectHTTPtoHTTPS")

    def test_AppSyncProtectedByWAF(self):
        self.go("AppSyncProtectedByWAF")

    def test_RDSEncryptionInTransit(self):
        self.go("RDSEncryptionInTransit")

    def test_ACMWildcardDomainName(self):
        self.go("ACMWildcardDomainName")

    def test_CloudfrontOriginNotHTTPSOnly(self):
        self.go("CloudfrontOriginNotHTTPSOnly")

    def test_LambdaOpenCorsPolicy(self):
        self.go("LambdaOpenCorsPolicy")

    def test_registry_load(self):
        registry = self.get_checks_registry()
        self.assertGreater(len(registry.checks), 0)

    def assert_evaluated_keys(self, expected_evaluated_keys: List[str], results: List[Record]):
        evaluated_keys_results = results[0].check_result["evaluated_keys"]
        self.assertCountEqual(expected_evaluated_keys, evaluated_keys_results)

    def create_report_from_graph_checks_results(self, checks_results, check):
        report = Report("cloudformation")
        first_results_key = list(checks_results.keys())[0]
        for check_result in checks_results[first_results_key]:
            entity = check_result["entity"]
            record = Record(check_id=check['id'],
                            check_name=check['name'],
                            check_result=copy.deepcopy(check_result),
                            code_block="",
                            file_path=entity.get(CustomAttributes.FILE_PATH),
                            file_line_range=[entity.get('__startline__'), entity.get('__endline__')],
                            resource=entity.get(CustomAttributes.BLOCK_NAME),
                            entity_tags=entity.get('tags', {}),
                            evaluations=None,
                            check_class=None,
                            file_abs_path=entity.get(CustomAttributes.FILE_PATH))
            if check_result["result"] == CheckResult.PASSED:
                report.passed_checks.append(record)
            if check_result["result"] == CheckResult.FAILED:
                report.failed_checks.append(record)
        return report


def wrap_policy(policy):
    policy['query'] = policy['definition']
    del policy['definition']



from aws_cdk import core
from aws_cdk import aws_elasticsearch as elasticsearch
from aws_cdk import aws_opensearchservice as opensearchservice

class MyElasticsearchDomainStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Elasticsearch Domain with LogPublishingOptions for different log types
        elasticsearch.CfnDomain(
            self, 'MyElasticsearchDomain',
            domain_name='my-elasticsearch-domain',
            elasticsearch_version='7.10',  # Replace with your desired Elasticsearch version
            node_to_node_encryption_options={
                'enabled': True
            },
            log_publishing_options={
                'logPublishingOptionsKey': elasticsearch.CfnDomain.LogPublishingOptionProperty(
                    cloud_watch_logs_log_group_arn='arn:aws:logs:REGION:ACCOUNT_ID:log-group:LOG_GROUP_NAME',
                    enabled=False
                )
            }
            # Other properties for your Elasticsearch Domain
        )

app = core.App()
MyElasticsearchDomainStack(app, "MyElasticsearchDomainStack")
app.synth()

class MyOpenSearchDomainStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define OpenSearch Service Domain with LogPublishingOptions for different log types
        opensearchservice.CfnDomain(
            self, 'MyOpenSearchDomain',
            domain_name='my-opensearch-domain',
            elasticsearch_version='7.10',  # Replace with your desired OpenSearch version
            node_to_node_encryption_options={
                'enabled': True
            },
            log_publishing_options={
                'logPublishingOptionsKey': opensearchservice.CfnDomain.LogPublishingOptionProperty(
                    cloud_watch_logs_log_group_arn='arn:aws:logs:REGION:ACCOUNT_ID:log-group:LOG_GROUP_NAME',
                    enabled=True
                )
            }
            # Other properties for your OpenSearch Service Domain
        )

app = core.App()
MyOpenSearchDomainStack(app, "MyOpenSearchDomainStack")
app.synth()


from aws_cdk import core
from aws_cdk import aws_logs as logs

class MyLogGroupStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define CloudWatch Logs Log Group with Retention Period
        logs.CfnLogGroup(
            self, 'MyLogGroup',
            log_group_name='my-log-group',
        )

app = core.App()
MyLogGroupStack(app, "MyLogGroupStack")
app.synth()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class LaunchConfigurationEBSEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = (
            "Ensure all data stored in the Launch configuration or instance Elastic Blocks Store "
            "is securely encrypted"
        )
        id = "CKV_AWS_8"
        supported_resources = ("aws_launch_configuration", "aws_instance")
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        """
            Looks for encryption configuration at launch configuration:
            https://www.terraform.io/docs/providers/aws/r/launch_configuration.html or
            https://www.terraform.io/docs/providers/aws/d/instance.html
        :param conf: aws_launch_configuration configuration
        :return: <CheckResult>
        """
        # for key in conf.keys():
        #     # If present, the encrypted flag will be determined by the snapshot
        #     # Note: checkov does not know if snapshot is encrypted, so we default to PASSED

        self.evaluated_keys = ["root_block_device"]
        root = conf.get("root_block_device")
        if not root or not root[0]:
            # Issue 496 - TF will create unencrypted EBS root by default if whole root_block_device block is omitted.
            return CheckResult.FAILED
        self.evaluated_keys.append("ebs_block_device")
        blocks = conf.get("ebs_block_device") or []

        all_blocks = root + blocks

        if not all_blocks:
            return CheckResult.UNKNOWN
        all_blocks_results = []

        for block in all_blocks:
            all_blocks_results.append(_is_block_encrypted(block))
        if CheckResult.FAILED in all_blocks_results:
            return CheckResult.FAILED
        elif CheckResult.UNKNOWN in all_blocks_results:
            return CheckResult.UNKNOWN
        else:
            return CheckResult.PASSED


check = LaunchConfigurationEBSEncryption()


def _is_block_encrypted(block) -> CheckResult:
    if isinstance(block, dict):
        if block.get("encrypted") in ([False], False) and not block.get("snapshot_id"):
            return CheckResult.FAILED
        elif block.get("encrypted") in ([True], True):
            return CheckResult.PASSED
        elif not block.get("encrypted") in ([True], True) and block.get("snapshot_id"):
            return CheckResult.PASSED
    return CheckResult.UNKNOWN



import os.path
import sys
from pathlib import Path
import xml.dom.minidom
from typing import List

import pytest
from pytest_mock import MockerFixture

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.output.csv import CSVSBOM
from checkov.common.output.cyclonedx import CycloneDX
from checkov.common.output.sarif import Sarif

EXAMPLES_DIR = Path(__file__).parent / "examples"
OUTPUTS_DIR = Path(__file__).parent / "outputs"


def _get_deterministic_items_in_cyclonedx(pretty_xml_as_list: List[str]) -> List[str]:
    # the lines with the fields "serialNumber", "bom-ref" and "timestamp" contain some not-deterministic data (uuids,
    # timestamp). so we skip these lines by the first 'if when checking whether we get the expected results
    # in addition also the line that display the checkov version may be changeable, so we skip it as well
    # (in the second 'if')
    filtered_list = []
    for i, line in enumerate(pretty_xml_as_list):
        if "bom-ref" not in line and "serialNumber" not in line and "timestamp" not in line:
            if i == 0 or not any(tool_name in pretty_xml_as_list[i - 1] for tool_name in
                                 ("<name>checkov</name>", "<name>cyclonedx-python-lib</name>")):
                filtered_list.append(line)
    return filtered_list


def test_console_output(mocker, sca_image_report):
    console_output = sca_image_report.print_console(False, False, None, None, False)
    assert console_output == "".join(
        [
            "sca_image scan results:\n",
            "\n",
            "Passed checks: 1, Failed checks: 3, Skipped checks: 1\n",
            "\n",
            "\t/path/to/Dockerfile (sha256:123456) - CVEs Summary:\n",
            '\t┌──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┐\n',
            '\t│ Total CVEs: 3        │ critical: 0          │ high: 0              │ medium: 1            │ low: 1               │ skipped: 1           │ Total Packages Used: │\n',
            "\t│                      │                      │                      │                      │                      │                      │ 0                    │\n",
            '\t├──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┤\n',
            '\t├──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┤\n',
            '\t│ Package              │ CVE ID               │ Severity             │ Current version      │ Root fixed version   │ Compliant version    │ Reachability         │\n',
            '\t├──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┤\n',
            '\t│ perl                 │ CVE-2020-16156       │ MEDIUM               │ 5.34.0-3ubuntu1      │ N/A                  │ N/A                  │                      │\n',
            '\t├──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┤\n',
            '\t│ pcre2                │ CVE-2022-1587        │ LOW                  │ 10.39-3build1        │ N/A                  │ N/A                  │                      │\n',
            '\t└──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┘\n',
            "\n",
            "\t/path/to/Dockerfile (sha256:123456) - Licenses Statuses:\n",
            '\t┌────────────────────────────────┬────────────────────────────────┬────────────────────────────────┬────────────────────────────────────┐\n',
            "\t│ Package name                   │ Package version                │ Policy ID                      │ License                            │\n",
            '\t├────────────────────────────────┼────────────────────────────────┼────────────────────────────────┼────────────────────────────────────┤\n',
            '\t│ perl                           │ 5.34.0-3ubuntu1                │ BC_LIC_1                       │ Apache-2.0-Fake                    │\n',
            '\t└────────────────────────────────┴────────────────────────────────┴────────────────────────────────┴────────────────────────────────────┘\n',
        ]
    )


def test_console_output_in_tty(mocker: MockerFixture, sca_image_report):
    # simulate a tty call by enforcing color
    mocker.patch.dict(os.environ, {"FORCE_COLOR": "True"})

    console_output = sca_image_report.print_console(False, False, None, None, False)

    # then
    assert console_output == "".join(
        [
            '\x1b[34msca_image scan results:\n',
            '\x1b[0m\x1b[36m\n',
            'Passed checks: 1, Failed checks: 3, Skipped checks: 1\n',
            '\n',
            '\x1b[0m\t/path/to/Dockerfile (sha256:123456) - CVEs Summary:\n',
            '\t┌──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┐\n',
            '\t│ Total CVEs: 3        │ critical: 0          │ high: 0              │ medium: 1            │ low: 1               │ skipped: 1           │ Total Packages Used: │\n',
            "\t│                      │                      │                      │                      │                      │                      │ 0                    │\n",
            '\t├──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┤\n',
            '\t├──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┬──────────────────────┤\n',
            '\t│ Package              │ CVE ID               │ Severity             │ Current version      │ Root fixed version   │ Compliant version    │ Reachability         │\n',
            '\t├──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┤\n',
            '\t│ perl                 │ CVE-2020-16156       │ MEDIUM               │ 5.34.0-3ubuntu1      │ N/A                  │ N/A                  │                      │\n',
            '\t├──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┼──────────────────────┤\n',
            '\t│ pcre2                │ CVE-2022-1587        │ LOW                  │ 10.39-3build1        │ N/A                  │ N/A                  │                      │\n',
            '\t└──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┴──────────────────────┘\n',
            '\n',
            '\t/path/to/Dockerfile (sha256:123456) - Licenses Statuses:\n',
            '\t┌────────────────────────────────┬────────────────────────────────┬────────────────────────────────┬────────────────────────────────────┐\n',
            "\t│ Package name                   │ Package version                │ Policy ID                      │ License                            │\n",
            '\t├────────────────────────────────┼────────────────────────────────┼────────────────────────────────┼────────────────────────────────────┤\n',
            '\t│ perl                           │ 5.34.0-3ubuntu1                │ BC_LIC_1                       │ Apache-2.0-Fake                    │\n',
            '\t└────────────────────────────────┴────────────────────────────────┴────────────────────────────────┴────────────────────────────────────┘\n',
        ]
    )


def test_get_cyclonedx_report(sca_image_report, tmp_path: Path):
    cyclonedx_reports = [sca_image_report]
    cyclonedx = CycloneDX(repo_id="bridgecrewio/example", reports=cyclonedx_reports)
    cyclonedx_output = cyclonedx.get_xml_output()
    dom = xml.dom.minidom.parseString(cyclonedx_output)
    pretty_xml_as_string = str(dom.toprettyxml())
    with open(os.path.join(OUTPUTS_DIR, "results_cyclonedx.xml")) as f_xml:
        expected_pretty_xml = f_xml.read()

    actual_pretty_xml_as_list = _get_deterministic_items_in_cyclonedx(pretty_xml_as_string.split("\n"))
    expected_pretty_xml_as_list = _get_deterministic_items_in_cyclonedx(expected_pretty_xml.split("\n"))

    assert actual_pretty_xml_as_list == expected_pretty_xml_as_list


def test_get_csv_report(sca_image_report, tmp_path: Path):
    file_name = "container_images.csv"
    csv_sbom_report = CSVSBOM()
    csv_sbom_report.add_report(report=sca_image_report, git_org="acme", git_repository="bridgecrewio/example")
    csv_sbom_report.persist_report_container_images(file_name=file_name, is_api_key=True, output_path=str(tmp_path))
    output_file_path = tmp_path / file_name
    csv_output = output_file_path.read_text()
    csv_output_str = csv_sbom_report.get_csv_output_packages(check_type=CheckType.SCA_IMAGE)

    # # then
    expected_csv_output = [
        'Package,Version,Path,Line(s),Git Org,Git Repository,Vulnerability,Severity,Description,Licenses,Fix Version,Registry URL,Root Package,Root Version',
        'perl,5.34.0-3ubuntu1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,CVE-2020-16156,MEDIUM,CPAN 2.28 allows Signature Verification Bypass.,Apache-2.0-Fake,N/A,,,',
        'pcre2,10.39-3build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,CVE-2022-1587,LOW,An out-of-bounds read vulnerability was discovered in the PCRE2 library in the get_recurse_data_length() function of the pcre2_jit_compile.c file. This issue affects recursions in JIT-compiled regular expressions caused by duplicate data transfers.,Apache-2.0,N/A,,,',
        'pcre2,10.39-3build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,CVE-2022-1586,LOW,An out-of-bounds read vulnerability was discovered in the PCRE2 library in the compile_xclass_matchingpath() function of the pcre2_jit_compile.c file. This involves a unicode property matching issue in JIT-compiled regular expressions. The issue occurs because the character was not fully read in case-less matching within JIT.,Apache-2.0,N/A,,,',
        'bzip2,1.0.8-5build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,,,,Unknown,N/A,,,',
        'libidn2,2.3.2-2build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,,,,Unknown,N/A,,,', '']

    csv_output_as_list = csv_output.split("\n")
    assert csv_output_as_list == expected_csv_output

    expected_csv_output_str = [
        'Package,Version,Path,Line(s),Git Org,Git Repository,Vulnerability,Severity,Description,Licenses,Fix Version,Registry URL,Root Package,Root Version',
        '"perl",5.34.0-3ubuntu1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,CVE-2020-16156,MEDIUM,"CPAN 2.28 allows Signature Verification Bypass.","Apache-2.0-Fake",N/A,,,',
        '"pcre2",10.39-3build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,CVE-2022-1587,LOW,"An out-of-bounds read vulnerability was discovered in the PCRE2 library in the get_recurse_data_length() function of the pcre2_jit_compile.c file. This issue affects recursions in JIT-compiled regular expressions caused by duplicate data transfers.","Apache-2.0",N/A,,,',
        '"pcre2",10.39-3build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,CVE-2022-1586,LOW,"An out-of-bounds read vulnerability was discovered in the PCRE2 library in the compile_xclass_matchingpath() function of the pcre2_jit_compile.c file. This involves a unicode property matching issue in JIT-compiled regular expressions. The issue occurs because the character was not fully read in case-less matching within JIT.","Apache-2.0",N/A,,,',
        '"bzip2",1.0.8-5build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,,,"","Unknown",N/A,,,',
        '"libidn2",2.3.2-2build1,/path/to/Dockerfile (sha256:123456),,acme,bridgecrewio/example,,,"","Unknown",N/A,,,',
        '']
    csv_output_str_as_list = csv_output_str.split("\n")
    assert csv_output_str_as_list == expected_csv_output_str


def test_sarif_output(sca_image_report_scope_function):
    # The creation of sarif_json may change the input report. in order not to affect the other tests, we use
    # a report that is unique for the scope of the function

    # given
    sarif_output = Sarif(reports=[sca_image_report_scope_function], tool="Checkov").json

    # then
    sarif_output["runs"][0]["tool"]["driver"]["version"] = "2.0.x"
    expected_sarif_json = {
        "$schema": "https://raw.githubusercontent.com/oasis-tcs/sarif-spec/main/sarif-2.1/schema/sarif-schema-2.1.0.json",
        "version": "2.1.0",
        "runs": [
            {
                "tool": {
                    "driver": {
                        "name": "Checkov",
                        "version": "2.0.x",
                        "informationUri": "https://checkov.io",
                        "rules": [
                            {
                                "id": "Apache-2.0-Fake_perl@5.34.0-3ubuntu1",
                                "name": "License Apache-2.0-Fake - perl: 5.34.0-3ubuntu1",
                                "shortDescription": {"text": "License Apache-2.0-Fake - perl: 5.34.0-3ubuntu1"},
                                "fullDescription": {"text": "Package perl@5.34.0-3ubuntu1 has license Apache-2.0-Fake"},
                                "help": {"text": "SCA license\nResource: path/to/Dockerfile (sha256:123456).perl"},
                                "defaultConfiguration": {"level": "error"},
                            },
                            {
                                "id": "CVE-2020-16156_perl@5.34.0-3ubuntu1",
                                "name": "CVE-2020-16156 - perl: 5.34.0-3ubuntu1",
                                "shortDescription": {"text": "CVE-2020-16156 - perl: 5.34.0-3ubuntu1"},
                                "fullDescription": {"text": "CPAN 2.28 allows Signature Verification Bypass."},
                                "help": {
                                    "text": "SCA package scan\nResource: path/to/Dockerfile (sha256:123456).perl\nStatus: needed"
                                },
                                "defaultConfiguration": {"level": "error"},
                                "properties": {"security-severity": "7.8"},
                                "helpUri": "https://people.canonical.com/~ubuntu-security/cve/2020/CVE-2020-16156",
                            },
                            {
                                "id": "CVE-2022-1587_pcre2@10.39-3build1",
                                "name": "CVE-2022-1587 - pcre2: 10.39-3build1",
                                "shortDescription": {"text": "CVE-2022-1587 - pcre2: 10.39-3build1"},
                                "fullDescription": {
                                    "text": "An out-of-bounds read vulnerability was discovered in the PCRE2 library in the get_recurse_data_length() function of the pcre2_jit_compile.c file. This issue affects recursions in JIT-compiled regular expressions caused by duplicate data transfers."
                                },
                                "help": {
                                    "text": "SCA package scan\nResource: path/to/Dockerfile (sha256:123456).pcre2\nStatus: needed"
                                },
                                "defaultConfiguration": {"level": "error"},
                                "properties": {"security-severity": "9.1"},
                                "helpUri": "https://people.canonical.com/~ubuntu-security/cve/2022/CVE-2022-1587",
                            },
                            {
                                "id": "CVE-2022-1586_pcre2@10.39-3build1",
                                "name": "CVE-2022-1586 - pcre2: 10.39-3build1",
                                "shortDescription": {"text": "CVE-2022-1586 - pcre2: 10.39-3build1"},
                                "fullDescription": {
                                    "text": "An out-of-bounds read vulnerability was discovered in the PCRE2 library in the compile_xclass_matchingpath() function of the pcre2_jit_compile.c file. This involves a unicode property matching issue in JIT-compiled regular expressions. The issue occurs because the character was not fully read in case-less matching within JIT."
                                },
                                "help": {
                                    "text": "SCA package scan\nResource: path/to/Dockerfile (sha256:123456).pcre2\nStatus: needed"
                                },
                                "defaultConfiguration": {"level": "error"},
                                "properties": {"security-severity": "9.1"},
                                "helpUri": "https://people.canonical.com/~ubuntu-security/cve/2022/CVE-2022-1586",
                            },
                        ],
                        "organization": "bridgecrew",
                    }
                },
                "results": [
                    {
                        "ruleId": "Apache-2.0-Fake_perl@5.34.0-3ubuntu1",
                        "ruleIndex": 0,
                        "level": "error",
                        "attachments": [],
                        "message": {"text": "License Apache-2.0-Fake - perl: 5.34.0-3ubuntu1"},
                        "locations": [
                            {
                                "physicalLocation": {
                                    "artifactLocation": {"uri": "path/to/Dockerfile"},
                                    "region": {
                                        "startLine": 1,
                                        "endLine": 1,
                                        "snippet": {"text": "perl: 5.34.0-3ubuntu1"},
                                    },
                                }
                            }
                        ],
                    },
                    {
                        "ruleId": "CVE-2020-16156_perl@5.34.0-3ubuntu1",
                        "ruleIndex": 1,
                        "level": "warning",
                        "attachments": [],
                        "message": {"text": "CVE-2020-16156 - perl: 5.34.0-3ubuntu1"},
                        "locations": [
                            {
                                "physicalLocation": {
                                    "artifactLocation": {"uri": "path/to/Dockerfile"},
                                    "region": {
                                        "startLine": 1,
                                        "endLine": 1,
                                        "snippet": {"text": "perl: 5.34.0-3ubuntu1"},
                                    },
                                }
                            }
                        ],
                    },
                    {
                        "ruleId": "CVE-2022-1587_pcre2@10.39-3build1",
                        "ruleIndex": 2,
                        "level": "note",
                        "attachments": [],
                        "message": {"text": "CVE-2022-1587 - pcre2: 10.39-3build1"},
                        "locations": [
                            {
                                "physicalLocation": {
                                    "artifactLocation": {"uri": "path/to/Dockerfile"},
                                    "region": {
                                        "startLine": 1,
                                        "endLine": 1,
                                        "snippet": {"text": "pcre2: 10.39-3build1"},
                                    },
                                }
                            }
                        ],
                    },
                    {
                        "ruleId": "CVE-2022-1586_pcre2@10.39-3build1",
                        "ruleIndex": 3,
                        "level": "note",
                        "attachments": [],
                        "message": {"text": "CVE-2022-1586 - pcre2: 10.39-3build1"},
                        "locations": [
                            {
                                "physicalLocation": {
                                    "artifactLocation": {"uri": "path/to/Dockerfile"},
                                    "region": {
                                        "startLine": 1,
                                        "endLine": 1,
                                        "snippet": {"text": "pcre2: 10.39-3build1"},
                                    },
                                }
                            }
                        ],
                        "suppressions": [{"kind": "external", "justification": "CVE-2022-1586 is skipped"}],
                    },
                ],
            }
        ],
    }

    assert sarif_output == expected_sarif_json


def test_get_junit_xml_string(sca_image_report):
    # given
    test_suites = [sca_image_report.get_test_suite()]

    #  when
    junit_xml_output = sca_image_report.get_junit_xml_string(test_suites)

    # then
    assert xml.dom.minidom.parseString(junit_xml_output).toprettyxml() == "\n".join(
        [
            '<?xml version="1.0" ?>',
            '<testsuites disabled="0" errors="0" failures="3" tests="5" time="0.0">',
            "\t",
            "\t",
            '\t<testsuite disabled="0" errors="0" failures="3" name="sca_image scan" skipped="1" tests="5" time="0">',
            "\t\t",
            "\t\t",
            '\t\t<testcase name="[NONE][BC_LIC_1] SCA license" classname="/path/to/Dockerfile (sha256:123456).path/to/Dockerfile (sha256:123456).pcre2" file="/path/to/Dockerfile (sha256:123456)"/>',
            "\t\t",
            "\t\t",
            '\t\t<testcase name="[NONE][BC_LIC_1] SCA license" classname="/path/to/Dockerfile (sha256:123456).path/to/Dockerfile (sha256:123456).perl" file="/path/to/Dockerfile (sha256:123456)">',
            "\t\t\t",
            "\t\t\t",
            '\t\t\t<failure type="failure" message="SCA license">',
            "Resource: path/to/Dockerfile (sha256:123456).perl",
            "File: /path/to/Dockerfile (sha256:123456): 0-0",
            "Guideline: None",
            "",
            "\t\t0 | perl: 5.34.0-3ubuntu1</failure>",
            "\t\t\t",
            "\t\t",
            "\t\t</testcase>",
            "\t\t",
            "\t\t",
            '\t\t<testcase name="[MEDIUM][CKV_CVE_2020_16156] SCA package scan" classname="/path/to/Dockerfile (sha256:123456).path/to/Dockerfile (sha256:123456).perl" file="/path/to/Dockerfile (sha256:123456)">',
            "\t\t\t",
            "\t\t\t",
            '\t\t\t<failure type="failure" message="SCA package scan">',
            "Resource: path/to/Dockerfile (sha256:123456).perl",
            "File: /path/to/Dockerfile (sha256:123456): 0-0",
            "Guideline: None",
            "",
            "\t\t0 | perl: 5.34.0-3ubuntu1</failure>",
            "\t\t\t",
            "\t\t",
            "\t\t</testcase>",
            "\t\t",
            "\t\t",
            '\t\t<testcase name="[LOW][CKV_CVE_2022_1587] SCA package scan" classname="/path/to/Dockerfile (sha256:123456).path/to/Dockerfile (sha256:123456).pcre2" file="/path/to/Dockerfile (sha256:123456)">',
            "\t\t\t",
            "\t\t\t",
            '\t\t\t<failure type="failure" message="SCA package scan">',
            "Resource: path/to/Dockerfile (sha256:123456).pcre2",
            "File: /path/to/Dockerfile (sha256:123456): 0-0",
            "Guideline: None",
            "",
            "\t\t0 | pcre2: 10.39-3build1</failure>",
            "\t\t\t",
            "\t\t",
            "\t\t</testcase>",
            "\t\t",
            "\t\t",
            '\t\t<testcase name="[LOW][CKV_CVE_2022_1586] SCA package scan" classname="/path/to/Dockerfile (sha256:123456).path/to/Dockerfile (sha256:123456).pcre2" file="/path/to/Dockerfile (sha256:123456)">',
            "\t\t\t",
            "\t\t\t",
            '\t\t\t<skipped type="skipped" message="CVE-2022-1586 is skipped"/>',
            "\t\t\t",
            "\t\t",
            "\t\t</testcase>",
            "\t\t",
            "\t",
            "\t</testsuite>",
            "\t",
            "",
            "</testsuites>",
            ""
        ]
    )



import time
import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.secrets.plugins.entropy_keyword_combinator import EntropyKeywordCombinator
from checkov.secrets.runner import Runner


class TestCombinatorPluginMultilineJson(unittest.TestCase):
    def setUp(self) -> None:
        self.plugin = EntropyKeywordCombinator()

    def test_multiline_keyword_password_report(self):
        test_file_path = Path(__file__).parent / "json_multiline/test-multiline-secrets.json"

        report = Runner().run(
            root_folder=None, files=[str(test_file_path)], runner_filter=RunnerFilter(framework=['secrets'])
        )
        self.assertEqual(len(report.failed_checks), 5)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])

    def test_non_multiline_pair_time_limit_creating_report(self):
        # given
        test_files = [str(Path(__file__).parent / "json_multiline/pomerium_compose.json")]
        runner = Runner()
        runner_filter = RunnerFilter(framework=['secrets'])

        # when
        start_time = time.time()
        report = runner.run(root_folder=None, files=test_files, runner_filter=runner_filter)
        end_time = time.time()

        # then
        assert end_time-start_time < 2  # assert the time limit is not too long for parsing long lines.
        self.assertEqual(len(report.failed_checks), 6)
        # None of the results is related to multiline scanning - all is detected even if multiline scanning is disabled.
        # This is a different result compared to same data in .yml file.
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])



import unittest

import hcl2

from checkov.terraform.checks.resource.aws.RedshiftClusterAllowVersionUpgrade import check
from checkov.common.models.enums import CheckResult


class TestRedshiftClusterAllowVersionUpgrade(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
               resource "aws_redshift_cluster" "default" {
                  cluster_identifier = "tf-redshift-cluster"
                  database_name      = "mydb"
                  master_username    = "foo"
                  master_password    = "Mustbe8characters"  # checkov:skip=CKV_SECRET_6 test secret
                  node_type          = "dc1.large"
                  cluster_type       = "single-node"
                  allow_version_upgrade = false
                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success_missing_attribute(self):
        hcl_res = hcl2.loads("""
                   resource "aws_redshift_cluster" "default" {
                      cluster_identifier = "tf-redshift-cluster"
                      database_name      = "mydb"
                      master_username    = "foo"
                      master_password    = "Mustbe8characters"
                      node_type          = "dc1.large"
                      cluster_type       = "single-node"
                    }
                   """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
               resource "aws_redshift_cluster" "default" {
                  cluster_identifier = "tf-redshift-cluster"
                  database_name      = "mydb"
                  master_username    = "foo"
                  master_password    = "Mustbe8characters"
                  node_type          = "dc1.large"
                  cluster_type       = "single-node"
                  allow_version_upgrade = true
                }
                """)
        resource_conf = hcl_res['resource'][0]['aws_redshift_cluster']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()
